﻿(function () {
    "use strict";

    var app = WinJS.Application;
    var nav = WinJS.Navigation;
    var ui = WinJS.UI;

    //WinJS.strictProcessing();

    var downloadInProgress = 0;

    UrzaGatherer.Tools.UpdateBackground = function () {
        var dynamicBackground = Windows.Storage.ApplicationData.current.roamingSettings.values["dynamicBackground"];
        if (!dynamicBackground || UrzaGatherer.Planes.length == 0)
            return;

        // Background
        var cardIndex = Math.floor(Math.random() * UrzaGatherer.Planes.length);
        var card = UrzaGatherer.Planes[cardIndex];
        UrzaGatherer.Imaging.CropAndDarkenImage(card, 0.6, function (uri) {
            var holder = document.querySelector("#backgroundHolder");
            holder.style.backgroundImage = "url('" + uri + "')";

            WinJS.Utilities.addClass(holder, "loaded");

            var legend = document.querySelector(".legend");

            if (WinJS.Utilities.hasClass(legend, "hidden")) {
                WinJS.Utilities.removeClass(legend, "hidden");
            }

            legend.innerText = card.name + " / " + card.type;
            legend.onclick = function () {
                nav.navigate("/pages/card/card.html", { cards: new WinJS.Binding.List([card]), cardIndex: 0 });
            }
        });
    };

    app.onactivated = function (eventObject) {
        if (eventObject.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.launch ||
            eventObject.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.search ||
            eventObject.detail.kind === Windows.ApplicationModel.Activation.ActivationKind.fileOpenPicker) {

            UrzaGatherer.Screen = {
                width: window.innerWidth,
                height: window.innerHeight
            };

            eventObject.setPromise(WinJS.UI.processAll().then(function () {
                WinJS.Resources.processAll().then(function () {

                    switch (eventObject.detail.kind) {
                        case Windows.ApplicationModel.Activation.ActivationKind.search:
                            UrzaGatherer.TextQuery = eventObject.detail.queryText.toLowerCase();
                            break;
                        case Windows.ApplicationModel.Activation.ActivationKind.fileOpenPicker:
                            UrzaGatherer.FileOpenPickerUI = eventObject.detail.fileOpenPickerUI;
                            break;
                        default:
                            if (UrzaGatherer.Blocks.length != 0) {
                                var expansionID = parseInt(eventObject.detail.arguments);

                                var expansion;

                                for (var index = 0; index < UrzaGatherer.Expansions.length; index++) {
                                    var exp = UrzaGatherer.Expansions[index];

                                    if (exp.id == expansionID) {
                                        expansion = exp;
                                        break;
                                    }
                                }
                                nav.navigate("/pages/expansion/expansion.html", { expansion: expansion });
                                return;
                            }
                            else {
                                UrzaGatherer.Arguments = eventObject.detail.arguments;
                            }
                            break;
                    }

                    // Settings
                    WinJS.Application.onsettings = function (e) {
                        e.detail.applicationcommands = {
                            "aboutDiv": { title: UrzaGatherer.Tools.GetString("About") },
                            "databaseDiv": { title: UrzaGatherer.Tools.GetString("Database") },
                            "settingsDiv": { title: UrzaGatherer.Tools.GetString("Settings") }
                        };
                        WinJS.UI.SettingsFlyout.populateSettings(e);
                    };

                    // Zoom range
                    var zoomRange = document.getElementById("zoomRange");

                    if (window.innerHeight > 1024) {
                        zoomRange.max = 100;
                        zoomRange.min = 20;
                    }
                    else {
                        zoomRange.max = 80;
                        zoomRange.min = 20;
                    }

                    var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];
                    if (!zoomLevel) {
                        zoomLevel = 50;
                        Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"] = 50;
                    }
                    zoomRange.value = zoomLevel;

                    zoomRange.addEventListener("MSPointerUp", function () {
                        Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"] = zoomRange.value;
                        Windows.Storage.ApplicationData.current.signalDataChanged();
                    });

                    // Offline mode
                    var on = document.getElementById("offlineModeOn");
                    var off = document.getElementById("offlineModeOff");
                    var offlineMode = Windows.Storage.ApplicationData.current.roamingSettings.values["offlineMode"];

                    if (offlineMode)
                        on.checked = true;
                    else
                        off.checked = true;

                    on.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["offlineMode"] = true;
                        UrzaGatherer.UpdateOfflineState();
                    });

                    off.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["offlineMode"] = false;
                        UrzaGatherer.UpdateOfflineState();
                    });

                    // Dynamic background
                    var englishCardText = document.getElementById("englishCardText");
                    var frenchCardText = document.getElementById("frenchCardText");

                    var cardText = Windows.Storage.ApplicationData.current.roamingSettings.values["cardText"];

                    if (cardText) {
                        if (cardText == "fr") {
                            frenchCardText.checked = true;
                        }
                        else {
                            englishCardText.checked = true;
                        }
                    }
                    else {
                        englishCardText.checked = true;
                    }

                    englishCardText.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["cardText"] = "en";
                        UrzaGatherer.UpdateLanguageState();
                    });

                    frenchCardText.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["cardText"] = "fr";
                        UrzaGatherer.UpdateLanguageState();
                    });

                    // Dynamic background
                    on = document.getElementById("dynamicBackgroundOn");
                    off = document.getElementById("dynamicBackgroundOff");
                    var cycleButton = document.querySelector(".cycleBackground");

                    var dynamicBackground = Windows.Storage.ApplicationData.current.roamingSettings.values["dynamicBackground"];

                    if (dynamicBackground)
                        on.checked = true;
                    else {
                        WinJS.Utilities.addClass(cycleButton, "hidden");
                        off.checked = true;

                        WinJS.Utilities.addClass(document.querySelector("#backgroundHolder"), "loaded");
                        document.querySelector(".legend").innerText = "";
                    }

                    on.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["dynamicBackground"] = true;
                        UrzaGatherer.Tools.UpdateBackground();
                        WinJS.Utilities.removeClass(cycleButton, "hidden");
                    });

                    off.addEventListener("change", function () {
                        Windows.Storage.ApplicationData.current.roamingSettings.values["dynamicBackground"] = false;
                        document.querySelector(".legend").innerText = "";
                        document.querySelector("#backgroundHolder").style.backgroundImage = "";
                        WinJS.Utilities.addClass(cycleButton, "hidden");
                    });

                    cycleButton.addEventListener("click", function () {
                        UrzaGatherer.Tools.UpdateBackground();
                    });

                    document.querySelector(".import").addEventListener("click", function () {
                        if (!UrzaGatherer.UserData) {
                            UrzaGatherer.Tools.ShowWarning("UserDataNotYetLoaded");
                            return;
                        }
                        var picker = new Windows.Storage.Pickers.FileOpenPicker();
                        picker.fileTypeFilter.append(".urza");
                        picker.fileTypeFilter.append(".txt");
                        picker.pickSingleFileAsync().then(function (file) {

                            if (file == null)
                                return;
                            WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");

                            UrzaGatherer.Collection.LoadFileLocally(file);

                            WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
                        });
                    });

                    document.querySelector(".export").addEventListener("click", function () {
                        if (!UrzaGatherer.UserData) {
                            UrzaGatherer.Tools.ShowWarning("UserDataNotYetLoaded");
                            return;
                        }
                        var picker = new Windows.Storage.Pickers.FileSavePicker();
                        picker.fileTypeChoices.insert(UrzaGatherer.Tools.GetString("UrzaFiles"), [".urza"]);
                        picker.pickSaveFileAsync().then(function (file) {
                            if (file == null)
                                return;
                            UrzaGatherer.Collection.SaveFileLocally(file).done();
                        });
                    });

                    // Stats
                    var stats = document.getElementById("stats");

                    WinJS.Namespace.define("UrzaGatherer.Stats", {
                        IncrementDownloads: function () {
                            downloadInProgress++;
                            WinJS.Utilities.removeClass(stats, "hidden");
                            stats.innerText = downloadInProgress + UrzaGatherer.Tools.GetString("DownloadInProgress");
                        },
                        DecrementDownloads: function () {
                            downloadInProgress--;
                            stats.innerText = downloadInProgress + UrzaGatherer.Tools.GetString("DownloadInProgress");
                            if (downloadInProgress == 0)
                                WinJS.Utilities.addClass(stats, "hidden");
                        }
                    });

                    // About
                    var packageId = Windows.ApplicationModel.Package.current.id;
                    var version = packageId.version;

                    document.getElementById("aboutVersion").innerText = UrzaGatherer.Tools.GetString("Version") + " " + version.major + "." + version.minor + "." + version.build + "." + version.revision;

                    // Search
                    var searchPane = Windows.ApplicationModel.Search.SearchPane.getForCurrentView();
                    searchPane.addEventListener("querysubmitted", function (e) {
                        nav.navigate("/pages/search/search.html", { textQuery: e.queryText.toLowerCase() });
                    }, false);

                    // Register to Handle Suggestion Request
                    searchPane.addEventListener("suggestionsrequested", function (e) {
                        var query = e.queryText;
                        var maxNumberOfSuggestions = 5;
                        var suggestionRequest = e.request;

                        for (var i = 0, len = UrzaGatherer.Cards.length; i < len; i++) {
                            if (UrzaGatherer.Cards[i].name.substr(0, query.length).toLowerCase() === query.toLowerCase()) {
                                suggestionRequest.searchSuggestionCollection.appendQuerySuggestion(UrzaGatherer.Cards[i].name);
                                if (suggestionRequest.searchSuggestionCollection.size === maxNumberOfSuggestions) {
                                    break;
                                }
                            }
                        }
                    }, false);

                    // Favorites
                    var favoritesListViewDiv = document.getElementById("favoritesList");
                    var favoritesListView = favoritesListViewDiv.winControl;

                    favoritesListView.itemDataSource = UrzaGatherer.FavoritesList.dataSource;

                    favoritesListView.forceLayout();

                    var oniteminvoked = function (eventObject) {
                        var item = eventObject.detail.itemPromise.then(function (invokedItem) {
                            nav.navigate("/pages/card/card.html", { cards: new WinJS.Binding.List([invokedItem.data]), cardIndex: 0 });
                        });
                    };

                    ui.setOptions(favoritesListView, {
                        oniteminvoked: oniteminvoked,
                        layout: new ui.GridLayout({
                            maxRows: 1, 
                            groupHeaderPosition: "top",
                            groupInfo: function () {
                                return {
                                    enableCellSpanning: true,
                                    cellWidth: 48,
                                    cellHeight: 68
                                };
                            }
                        })
                    });

                    if (nav.location) {
                        nav.history.current.initialPlaceholder = true;
                        return nav.navigate(nav.location, nav.state);
                    } else {
                        return nav.navigate(UrzaGatherer.navigator.home);
                    }
                });
            }));
        }
    };

    app.oncheckpoint = function (args) {
    };

    app.start();
})();
