﻿(function () {
    "use strict";

    var blocksList = new WinJS.Binding.List();
    var expansionsList = new WinJS.Binding.List();
    var expansionsListWithCardOfTheDay = new WinJS.Binding.List();
    var favoritesList = new WinJS.Binding.List();
    var blocks = [];
    var expansions = [];
    var cards = [];
    var root = "http://www.urzagatherer.com";
    var planes = [];
    var cardOfTheDayBinding;

    var expansionSorter = function (i0, i1) {
        if (i0.orderInBlock > i1.orderInBlock)
            return 1;
        else if (i0.orderInBlock < i1.orderInBlock)
            return -1;
        return 0;
    };

    var updateCardOfTheDay = function () {
        var card;

        do {
            var cardNumber = Math.floor(Math.random() * cards.length);
            card = cards[cardNumber];
        }
        while (card.size.isPlane);

        if (cardOfTheDayBinding.element) {
            WinJS.Utilities.query("img", cardOfTheDayBinding.element).forEach(function (img) {
                WinJS.Utilities.removeClass(img, "loaded");
            });
        }

        cardOfTheDayBinding.url = card.logo;
        cardOfTheDayBinding.name = card.name;
        cardOfTheDayBinding.card = card;
    }

    var updateCardLanguageState = function (card, useFrench) {
        if (useFrench) {
            card.name = card.nameFr;
            card.type = card.typeFr;
            card.text = card.textFr;
            card.flavor = card.flavorFr;
            card.multiverseId = card.multiverseIdFr;

            switch (card.rarityId) {
                case 1:
                    card.rarity = "Commune";
                    break;
                case 2:
                    card.rarity = "Peu commune";
                    break;
                case 3:
                    card.rarity = "Rare";
                    break;
                case 4:
                    card.rarity = "Mythique";
                    break;
            }

            switch (card.colorId) {
                case 1:
                    card.color = "Noire";
                    break;
                case 2:
                    card.color = "Rouge";
                    break;
                case 3:
                    card.color = "Bleue";
                    break;
                case 4:
                    card.color = "Blanche";
                    break;
                case 5:
                    card.color = "Verte";
                    break;
                case 6:
                    card.color = "Artefact";
                    break;
                case 7:
                    card.color = "Terrain";
                    break;
                case 8:
                    card.color = "Multi-couleurs";
                    break;
                case 10:
                    card.color = "Eldrazi";
                    break;
            }
        }
        else {
            switch (card.rarityId) {
                case 1:
                    card.rarity = "Common";
                    break;
                case 2:
                    card.rarity = "Uncommon";
                    break;
                case 3:
                    card.rarity = "Rare";
                    break;
                case 4:
                    card.rarity = "Mythic rare";
                    break;
            }

            switch (card.colorId) {
                case 1:
                    card.color = "Black";
                    break;
                case 2:
                    card.color = "Red";
                    break;
                case 3:
                    card.color = "Blue";
                    break;
                case 4:
                    card.color = "White";
                    break;
                case 5:
                    card.color = "Green";
                    break;
                case 6:
                    card.color = "Artifact";
                    break;
                case 7:
                    card.color = "Land";
                    break;
                case 8:
                    card.color = "Multicolors";
                    break;
                case 10:
                    card.color = "Eldrazi";
                    break;
            }
        }

        if (!useFrench || !card.name) {
            card.name = card.nameEn;
        }

        if (!useFrench || !card.type) {
            card.type = card.typeEn;
        }

        if (!useFrench || !card.text) {
            card.text = card.textEn;
        }

        if (!useFrench || !card.flavor) {
            card.flavor = card.flavorEn;
        }

        if (!useFrench || !card.multiverseId) {
            card.multiverseId = card.multiverseIdEn;
        }

        if (!card.text)
            card.text = "N/A";

        if (!card.flavor)
            card.flavor = "N/A";
    }

    var updateLanguageState = function () {
        var useFrench = false;

        var cardText = Windows.Storage.ApplicationData.current.roamingSettings.values["cardText"];

        if (cardText) {
            if (cardText == "fr") {
                useFrench = true;
            }
        }

        for (var cardIndex = 0; cardIndex < cards.length; cardIndex++) {
            var card = cards[cardIndex];

            updateCardLanguageState(card, useFrench);
        }
    }

    var updateBlockOfflineState = function (block, offline) {
            if (offlineMode) {
                block.logo = "ms-appdata:///local/blocks/" + block.name.replace(":", "_") + ".png";
                block.banner = "ms-appdata:///local/blocks/" + block.name.replace(":", "_") + "_banner.png";
            }
            else {
                block.logo = root + "/blocks/" + block.name.replace(":", "_") + ".png";
                block.banner = root + "/blocks/" + block.name.replace(":", "_") + "_banner.png";
            }
    }

    var updateExpansionOfflineState = function (expansion, offline) {
        if (offlineMode) {
            expansion.logo = "ms-appdata:///local/logos/" + expansion.name.replace(":", "_") + ".png";
            expansion.banner = "ms-appdata:///local/expansions/" + expansion.name.replace(":", "_") + ".png";
        }
        else {
            expansion.logo = root + "/logos/" + expansion.name.replace(":", "_") + ".png";
            expansion.banner = root + "/expansions/" + expansion.name.replace(":", "_") + ".png";
        }
    }

    var updateCardOfflineState = function (card, offline) {
        if (!card.alt)
            return;

        if (offlineMode) {
            card.logo = "ms-appdata:///local/" + card.alt.replace("\\", "/");
        }
        else {
            card.logo = root + "/cards/" + card.alt.replace("\\", "/");
        }
    }

    var updateOfflineState = function () {
        var offlineMode = Windows.Storage.ApplicationData.current.roamingSettings.values["offlineMode"];
        
        for (var blockIndex = 0; blockIndex < blocks.length; blockIndex++) {
            var block = blocks[blockIndex];

            updateBlockOfflineState(block, offlineMode);
        }

        for (var expansionIndex = 0; expansionIndex < expansions.length; expansionIndex++) {
            var expansion = expansions[expansionIndex];

            updateExpansionOfflineState(expansion);
        }

        for (var cardIndex = 0; cardIndex < cards.length; cardIndex++) {
            var card = cards[cardIndex];
            updateCardOfflineState(card, offlineMode);
        }
    }

    var processBlocks = function (data) {
        // Parameters
        var useFrench = false;

        var cardText = Windows.Storage.ApplicationData.current.roamingSettings.values["cardText"];

        if (cardText) {
            if (cardText == "fr") {
                useFrench = true;
            }
        }

        // JSON
        var result = JSON.parse(data);
        var offlineMode = Windows.Storage.ApplicationData.current.roamingSettings.values["offlineMode"];


        var incrementCompletion = function () {
            this.completionCount++;
            this.completionLevel = Math.floor((this.completionCount * 100) / this.cards.length) + "%";

            this.isChecked = (this.completionCount === this.cards.length);
            this.block.updateCheckedState();
        };

        var decrementCompletion = function () {
            this.completionCount--;
            this.completionLevel = Math.floor((this.completionCount * 100) / this.cards.length) + "%";

            this.isChecked = (this.completionCount === this.cards.length);
            this.block.updateCheckedState();
        };

        var updateCheckedState = function () {
            for (var index = 0; index < this.expansions.length; index++) {
                if (!this.expansions[index].isChecked) {
                    this.isChecked = false;
                    return;
                }
            }
            this.isChecked = true;
        };

        for (var blockIndex = 0; blockIndex < result.length; blockIndex++) {
            var block = result[blockIndex];

            if (block.name == "Oracle")
                continue;

            block.isChecked = false;

            block.updateCheckedState = updateCheckedState;
            updateBlockOfflineState(block, offlineMode);

            blocksList.push(block);
            blocks.push(block);

            var sortedExpansions = block.expansions.sort(expansionSorter);

            for (var expansionIndex = 0; expansionIndex < sortedExpansions.length; expansionIndex++) {
                var expansion = sortedExpansions[expansionIndex];
                expansion.block = block;
                expansion.completionCount = 0;
                expansion.completionLevel = "0%";
                expansion.isChecked = false;

                expansion.incrementCompletion = incrementCompletion;

                expansion.decrementCompletion = decrementCompletion;

                expansion.logoPath = "logos\\" + expansion.name.replace(":", "_") + ".png";

                updateExpansionOfflineState(expansion, offlineMode);

                expansionsList.push(expansion);
                expansionsListWithCardOfTheDay.push(expansion);
                expansions.push(expansion);

                expansion.cardsList = new WinJS.Binding.List();

                var price = 0;

                for (var cardIndex = 0; cardIndex < expansion.cards.length; cardIndex++) {
                    var card = expansion.cards[cardIndex];

                    card.expansion = expansion;
                    card.isChecked = false;

                    card.size = {
                        isPlane: (card.typeEn.substring(0, 7) == "Plane -"),
                        isScheme: (card.typeEn == "Scheme")
                    }

                    if (card.size.isPlane)
                        planes.push(card);

                    if (card.picturePath) {
                        card.alt = card.picturePath.replace("’", "'");
                        card.distantLogo = root + "/cards/" + card.alt.replace("\\", "/");
                    }

                    if (!card.power)
                        card.power = "N/A";

                    if (!card.manaCost) {
                        card.manaCost = "N/A";
                    }

                    if (!card.convertedManaCost) {
                        card.convertedManaCost = 0;
                    }

                    updateCardOfflineState(card, offlineMode);
                    updateCardLanguageState(card, useFrench);

                    var bindableCard = new WinJS.Binding.as(card);
                    expansion.cardsList.push(bindableCard);
                    cards.push(bindableCard);

                    price += card.price;
                }

                expansion.totalPrice = Math.round(price * 100) / 100 + "€";
                expansion.cardsCount = expansion.cards.length;
            }
        }

        // Card of the day
        cardOfTheDayBinding = new WinJS.Binding.as({ url: "", name: "" });

        updateCardOfTheDay();

        expansionsListWithCardOfTheDay.push(cardOfTheDayBinding);

        window.setInterval(updateCardOfTheDay, 20000);
    }

    var getBlocksDistant = function (totalSize, onload, onprogress) {
        var localFolder = Windows.Storage.ApplicationData.current.localFolder;
        var requestStr = root + "/urzaGatherer.json";

        var previousUpdate = (new Date).getTime();

        WinJS.xhr({
            url: requestStr + "?" + Math.random()
        }).then(function (request) {
            if (onprogress)
                onprogress(100);

            processBlocks(request.responseText);

            localFolder.createFileAsync("all.json", Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                Windows.Storage.FileIO.writeTextAsync(file, request.responseText);
            });

            if (onload)
                onload();
        }, function (err) {
            var msg = new Windows.UI.Popups.MessageDialog(UrzaGatherer.Tools.GetString("ErrorConnection"));
            msg.showAsync().then(function () {
                window.close();
                if (onload)
                    onload();
            });
        }, function (progress) {
            if (onprogress) {

                var currentUpdate = (new Date).getTime();
                if (currentUpdate - previousUpdate < 250) // Do not overload the UI
                    return;

                previousUpdate = currentUpdate;

                var downloadedSize = progress.responseText.length;
                var current = Math.round((downloadedSize * 100) / totalSize);

                onprogress(current);
            }
        });
    }

    var getBlocksLocal = function (onload, onprogress) {
        var localFolder = Windows.Storage.ApplicationData.current.localFolder;

        localFolder.getFileAsync("all.json").done(function (file) {
            return Windows.Storage.FileIO.readTextAsync(file).then(function (data) {
                processBlocks(data);

                if (onload)
                    onload();
            });
        }, function () {
            Windows.Storage.ApplicationData.current.localSettings.values["baseInfo"] = "0";
            getBlocks(onload, onprogress);
        });
    };

    var getBlocks = function (onload, onprogress, setUpdateText) {
        // Check server version
        WinJS.xhr(
        {
            method: "GET",
            url: root + "/versionSize.txt?" + Math.random()
        }).then(function (request) {
            var serverVersion = request.responseText;
            var localVersion = Windows.Storage.ApplicationData.current.localSettings.values["baseInfo"];

            if (!localVersion || localVersion != serverVersion) {

                WinJS.xhr(
                {
                    method: "GET",
                    url: root + "/versionNews.txt?" + Math.random()
                }).then(function (request) {

                    setUpdateText(JSON.parse(request.responseText));

                    getBlocksDistant(parseInt(serverVersion), function () {
                        Windows.Storage.ApplicationData.current.localSettings.values["baseInfo"] = serverVersion;
                        onload();
                    }, onprogress);
                });
                return;
            }

            // Local?
            getBlocksLocal(onload, onprogress);
        }, function (err) {
            getBlocksLocal(onload, onprogress);
        });

    };

    var updateCardsStatus = function () {
        for (var index = 0; index < cards.length; index++) {
            var card = cards[index];

            if (UrzaGatherer.UserData && UrzaGatherer.UserData[card.id]) {
                if (!card.isChecked) {
                    card.expansion.incrementCompletion();
                    card.isChecked = true;
                }
            }
            else {
                if (card.isChecked) {
                    card.isChecked = false;
                    card.expansion.decrementCompletion();
                }
            }
        }
        favoritesList.splice(0, favoritesList.length);
        if (UrzaGatherer.UserData && UrzaGatherer.UserData.favorites) {
            for (var favoriteIndex = 0; favoriteIndex < UrzaGatherer.UserData.favorites.length; favoriteIndex++) {
                var cardId = UrzaGatherer.UserData.favorites[favoriteIndex];

                for (var index = 0; index < cards.length; index++) {
                    var card = cards[index];
                    if (card.id == cardId) {
                        favoritesList.push(card);
                        break;
                    }
                }
            }
        }
    };

    var getStatistics = function (list) {
        var checked = 0;
        var totalPrice = 0;
        var checkedPrice = 0;

        var len = (list ? list.length : cards.length);

        if (len == 0) {
            return {
                cards: "",
                price: ""
            };
        }

        for (var index = 0; index < len; index++) {
            var card = (list ? list.getAt(index) : cards[index]);

            if (card.isChecked == true) {
                checked++;
                checkedPrice += card.price;
            }

            totalPrice += card.price;
        }

        return {
            cards: checked + " / " + len + UrzaGatherer.Tools.GetString("Cards") + " (" + (Math.round(((checked * 100) / len) * 100) / 100) + "%)",
            price: (Math.round(checkedPrice * 100) / 100) + "€ / " + (Math.round(totalPrice * 100) / 100) + "€"
        };
    }

    WinJS.Namespace.define("UrzaGatherer", {
        Blocks: blocksList,
        ExpansionsList: expansionsList,
        ExpansionsListWithCardOfTheDay: expansionsListWithCardOfTheDay,
        FavoritesList: favoritesList,
        Expansions: expansions,
        Cards: cards,
        Planes: planes,
        Init: getBlocks,
        Root: root,
        UpdateCardsStatus: updateCardsStatus,
        UpdateOfflineState: updateOfflineState,
        UpdateLanguageState: updateLanguageState,
        GetStatistics: getStatistics
    });
})();