﻿(function () {
    var itemRenderer = function (itemPromise) {
        return itemPromise.then(function (currentItem, recycled) {
            var template = document.querySelector(".itemTemplate").winControl.renderItem(itemPromise, recycled);
            template.renderComplete = template.renderComplete.then(function (elem) {

                if (currentItem.data.size.isPlane) {
                    var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];

                    if (!zoomLevel)
                        return;

                    var level = zoomLevel / 100.0;
                    elem.querySelector(".item-container").style.width = (480 * level * 2) + "px";
                }
            });
            return template.element;
        })
    };

    var prepareAdvancedFilters = function (list, onPointerUpFunction) {
        var maxPrice = 0.0;
        var maxForce = 0;
        var maxDefense = 0;
        var maxManaCost = 0;
        var minPrice = 100000000;
        var minForce = 100000000;
        var minDefense = 100000000;
        var minManaCost = 100000000;

        for (var index = 0; index < list.length ; index++) {
            var card = list.getAt(index);

            if (card.price) {
                if (card.price > maxPrice) {
                    maxPrice = card.price;
                } else if (card.price < minPrice) {
                    minPrice = card.price;
                }
            }

            if (card.force) {
                if (card.force > maxForce) {
                    maxForce = card.force;
                } else if (card.force < minForce) {
                    minForce = card.force;
                }
            }

            if (card.defense) {
                if (card.defense > maxDefense) {
                    maxDefense = card.defense;
                } else if (card.defense < minDefense) {
                    minDefense = card.defense;
                }
            }

            if (card.convertedManaCost > maxManaCost) {
                maxManaCost = card.convertedManaCost;
            } else if (card.convertedManaCost < minManaCost) {
                minManaCost = card.convertedManaCost;
            }
        }

        if (minPrice == 100000000)
            minPrice = 0;

        if (minForce == 100000000)
            minForce = 0;

        if (minDefense == 100000000)
            minDefense = 0;

        if (minManaCost == 100000000)
            minManaCost = 0;

        // Synchonize advanced filters
        var minimumManaCost = document.getElementById("minimumManaCost");
        var minimumPrice = document.getElementById("minimumPrice");
        var minimumForce = document.getElementById("minimumForce");
        var minimumDefense = document.getElementById("minimumDefense");
        var maximumManaCost = document.getElementById("maximumManaCost");
        var maximumPrice = document.getElementById("maximumPrice");
        var maximumForce = document.getElementById("maximumForce");
        var maximumDefense = document.getElementById("maximumDefense");

        minimumManaCost.max = maxManaCost;
        minimumPrice.max = Math.floor(maxPrice);
        minimumForce.max = maxForce;
        minimumDefense.max = maxDefense;
        maximumManaCost.max = maxManaCost;
        maximumPrice.max = Math.ceil(maxPrice);
        maximumForce.max = maxForce;
        maximumDefense.max = maxDefense;

        minimumManaCost.min = minManaCost;
        minimumPrice.min = Math.floor(minPrice);
        minimumForce.min = minForce;
        minimumDefense.min = minDefense;
        maximumManaCost.min = minManaCost;
        maximumPrice.min = Math.ceil(minPrice);
        maximumForce.min = minForce;
        maximumDefense.min = minDefense;

        minimumManaCost.value = minManaCost;
        minimumPrice.value = minimumPrice.min;
        minimumForce.value = minForce;
        minimumDefense.value = minDefense;
        maximumManaCost.value = maxManaCost;
        maximumPrice.value = maximumPrice.max;
        maximumForce.value = maxForce;
        maximumDefense.value = maxDefense;

        minimumManaCost.onmspointerup = function () { onPointerUpFunction(); };
        minimumPrice.onmspointerup = function () { onPointerUpFunction(); };
        minimumForce.onmspointerup = function () { onPointerUpFunction(); };
        minimumDefense.onmspointerup = function () { onPointerUpFunction(); };
        maximumManaCost.onmspointerup = function () { onPointerUpFunction(); };
        maximumPrice.onmspointerup = function () { onPointerUpFunction(); };
        maximumForce.onmspointerup = function () { onPointerUpFunction(); };
        maximumDefense.onmspointerup = function () { onPointerUpFunction(); };
    }

    var getCurrentAdvancedFiltersState = function () {
        var minimumManaCost = document.getElementById("minimumManaCost");
        var minimumPrice = document.getElementById("minimumPrice");
        var minimumForce = document.getElementById("minimumForce");
        var minimumDefense = document.getElementById("minimumDefense");
        var maximumManaCost = document.getElementById("maximumManaCost");
        var maximumPrice = document.getElementById("maximumPrice");
        var maximumForce = document.getElementById("maximumForce");
        var maximumDefense = document.getElementById("maximumDefense");

        return {
            currentMaxPrice: parseInt(maximumPrice.value),
            currentMinPrice: parseInt(minimumPrice.value),
            currentMaxManaCost: parseInt(maximumManaCost.value),
            currentMinManaCost: parseInt(minimumManaCost.value),
            currentMaxForce: parseInt(maximumForce.value),
            currentMinForce: parseInt(minimumForce.value),
            currentMaxDefense: parseInt(maximumDefense.value),
            currentMinDefense: parseInt(minimumDefense.value),
            priceFilterActivated: minimumPrice.value != minimumPrice.min || maximumPrice.value != maximumPrice.max,
            manaCostFilterActivated: minimumManaCost.value != minimumManaCost.min || maximumManaCost.value != maximumManaCost.max,
            forceFilterActivated: minimumForce.value != minimumForce.min || maximumForce.value != maximumForce.max,
            defenseFilterActivated: minimumDefense.value != minimumDefense.min || maximumDefense.value != maximumDefense.max
        }
    }

    var sortFunction = function (i0, i1) {
        var order = document.getElementById("orderSelect");

        switch (order.selectedIndex) {
            case 0:
                if (i0.number == i1.number) {
                    if (i0.tag && i1.tag)
                        return i0.tag > i1.tag;
                    return 0;
                }
                return i0.number - i1.number;
            case 1:
                return i0.name > i1.name;
        }
    }

    var getCurrentTextFilter = function (text) {
        var nameFilter = UrzaGatherer.Tools.GetString("NameFilter");
        var textFilter = UrzaGatherer.Tools.GetString("TextFilter");
        var flavorFilter = UrzaGatherer.Tools.GetString("FlavorFilter");
        var typeFilter = UrzaGatherer.Tools.GetString("TypeFilter");

        if (text.substring(0, nameFilter.length) === nameFilter) {
            return {
                onlyName: true,
                filter: text.replace(nameFilter, "")
            }
        }

        if (text.substring(0, textFilter.length) === textFilter) {
            return {
                onlyText: true,
                filter: text.replace(textFilter, "")
            }
        }

        if (text.substring(0, flavorFilter.length) === flavorFilter) {
            return {
                onlyFlavor: true,
                filter: text.replace(flavorFilter, "")
            }
        }

        if (text.substring(0, typeFilter.length) === typeFilter) {
            return {
                onlyType: true,
                filter: text.replace(typeFilter, "")
            }
        }

        return {
            filter: text
        }
    }

    var checkTextFilter = function (textFilter, item) {
        if (!textFilter)
            return true;

        var filter = textFilter.filter;

        if (textFilter.onlyName) {
            if ((item.name.toLowerCase().indexOf(filter) == -1))
                return false;

            return true;
        }

        if (textFilter.onlyText) {
            if (!item.text || (item.text.toLowerCase().indexOf(filter) == -1))
                return false;

            return true;
        }

        if (textFilter.onlyFlavor) {
            if (!item.flavor || (item.flavor.toLowerCase().indexOf(filter) == -1))
                return false;

            return true;
        }

        if (textFilter.onlyType) {
            if (!item.type || (item.type.toLowerCase().indexOf(filter) == -1))
                return false;

            return true;
        }

        if (!(item.name.toLowerCase().indexOf(filter) != -1
                || (item.text && item.text.toLowerCase().indexOf(filter) != -1)
                || (item.flavor && item.flavor.toLowerCase().indexOf(filter) != -1)
                || (item.type && item.type.toLowerCase().indexOf(filter) != -1)
                ))
            return false;

        return true;
    }


    WinJS.Namespace.define("UrzaGatherer.Common", {
        ItemRenderer: itemRenderer,
        PrepareAdvancedFilters: prepareAdvancedFilters,
        GetCurrentAdvancedFiltersState: getCurrentAdvancedFiltersState,
        SortFunction: sortFunction,
        GetCurrentTextFilter: getCurrentTextFilter,
        CheckTextFilter: checkTextFilter
    });
})();