﻿(function () {
    "use strict";

    var Imaging = Windows.Graphics.Imaging;

    var cropAndDarkenImage = function (card, amplification, then) {
        var image = new Image();

        var rect = getCardPictureRect(card);

        // lors du chargement
        image.addEventListener('load', function () {
            var canvas = document.createElement('canvas');

            canvas.width = UrzaGatherer.Screen.width / 4;
            canvas.height = UrzaGatherer.Screen.height / 4;

            if (!card.size.isPlane) {
                var ratio = rect.width / canvas.width;
                rect.height = canvas.height * ratio;
            }
            else {
                var ratio = rect.height / canvas.height;
                rect.width = canvas.width * ratio;
            }

            var context = canvas.getContext('2d');

            // Scale & crop
            context.drawImage(this, rect.left, rect.top, rect.width, rect.height, 0, 0, canvas.width, canvas.height);

            // Grayscale
            var imageData = context.getImageData(0, 0, canvas.width, canvas.height);
            var pixels = imageData.data;
            for (var index = 0; index < pixels.length; index += 4) {
                var r = pixels[index];
                var g = pixels[index + 1];
                var b = pixels[index + 2];

                var luminance = Math.floor(r * 0.3 + g * 0.59 + b * 0.11);

                luminance += (255 - luminance) * amplification;

                if (luminance > 255)
                    luminance = 255;

                pixels[index] = luminance;
                pixels[index + 1] = luminance;
                pixels[index + 2] = luminance;
            }

            context.putImageData(imageData, 0, 0);

            var blob = canvas.msToBlob();
            var url = URL.createObjectURL(blob);
            then(url);
        }, false);

        // Chargement
        if (card.logo.substring(0, 6) == "ms-app") {

            var source = card.logo.replace("ms-appdata:///local/", "").replace("/", "\\");
            Windows.Storage.ApplicationData.current.localFolder.getFileAsync(source).then(function () {
                image.src = card.logo;
            }, function () {
                image.src = card.distantLogo;
            });
        }
        else
            image.src = card.logo;
    }

    var getCardPictureRect = function (card) {
        if (card.size.isScheme) {
            return { left: 150, top: 170, width: 885, height: 960 };
        }

        if (card.size.isPlane) {
            return { left: 140, top: 140, width: 1210, height: 525 };
        }

        return { left: 45, top: 70, width: 390, height: 300 };
    }

    var rescaleImage = function (src, destinationWidth, destinationHeight, localfilename, fillAlpha, then) {
        var image = new Image();

        // lors du chargement
        image.addEventListener('load', function () {
            var canvas = document.createElement('canvas');

            canvas.width = destinationWidth;
            canvas.height = destinationHeight;

            var targetWidth;
            var targetHeight;

            if (this.width > this.height) {
                var ratio = destinationWidth / this.width;
                targetWidth = destinationWidth;
                targetHeight = this.height * ratio;
            }
            else {
                var ratio = destinationHeight / this.height;
                targetWidth = this.width * ratio;
                targetHeight = destinationHeight;
            }

            var context = canvas.getContext('2d');
            if (fillAlpha)
                context.clearRect(0, 0, canvas.width, canvas.height);
            else {
                context.fillStyle = "#fff";
                context.fillRect(0, 0, canvas.width, canvas.height);
            }
            context.drawImage(this, (canvas.width - targetWidth) / 2, (canvas.height - targetHeight) / 2, targetWidth, targetHeight);

            Windows.Storage.ApplicationData.current.localFolder.createFileAsync(localfilename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                file.openAsync(Windows.Storage.FileAccessMode.readWrite).then(function (stream) {

                    Imaging.BitmapEncoder.createAsync(Imaging.BitmapEncoder.pngEncoderId, stream).then(function (encoder) {
                        encoder.setPixelData(Imaging.BitmapPixelFormat.rgba8, Imaging.BitmapAlphaMode.straight,
                            canvas.width, canvas.height, 96, 96, new Uint8Array(context.getImageData(0, 0, canvas.width, canvas.height).data));
                        encoder.flushAsync().then(function () {

                            stream.flushAsync().then(function () {
                                stream.close();

                                if (then)
                                    then("ms-appdata:///local/" + localfilename.replace("\\", "/"));
                            });
                        });
                    });
                });
            });
        }, false);

        // Chargement
        image.src = src;
    }


    var rescaleAndCropImage = function (card, fileWidth, fileHeight, localfilename, fillAlpha, then) {
        var image = new Image();

        // lors du chargement
        image.addEventListener('error', function (e) {
            image.src = card.distantLogo;
        });

        image.addEventListener('load', function () {
            var canvas = document.createElement('canvas');

            canvas.width = fileWidth;
            canvas.height = fileHeight;

            var rect = getCardPictureRect(card);

            if (!card.size.isPlane) {
                var ratio = rect.width / canvas.width;
                rect.height = canvas.height * ratio;
            }
            else {
                var ratio = rect.height / canvas.height;
                rect.width = canvas.width * ratio;
            }

            var context = canvas.getContext('2d');
            if (fillAlpha)
                context.clearRect(0, 0, canvas.width, canvas.height);
            else {
                context.fillStyle = "#fff";
                context.fillRect(0, 0, canvas.width, canvas.height);
            }

            context.drawImage(this, rect.left, rect.top, rect.width, rect.height, 0, 0, canvas.width, canvas.height);

            Windows.Storage.ApplicationData.current.localFolder.createFileAsync(localfilename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                file.openAsync(Windows.Storage.FileAccessMode.readWrite).then(function (stream) {

                    Imaging.BitmapEncoder.createAsync(Imaging.BitmapEncoder.pngEncoderId, stream).then(function (encoder) {
                        encoder.setPixelData(Imaging.BitmapPixelFormat.rgba8, Imaging.BitmapAlphaMode.straight,
                            canvas.width, canvas.height, 96, 96, new Uint8Array(context.getImageData(0, 0, canvas.width, canvas.height).data));
                        encoder.flushAsync().then(function () {

                            stream.flushAsync().then(function () {
                                stream.close();

                                if (then)
                                    then("ms-appdata:///local/" + localfilename.replace("\\", "/"));
                            });
                        });
                    });
                });
            });
        }, false);

        // Chargement
        image.src = card.logo;
    }

    WinJS.Namespace.define("UrzaGatherer.Imaging", {
        RescaleImage: rescaleImage,
        CropAndDarkenImage: cropAndDarkenImage,
        RescaleAndCropImage: rescaleAndCropImage
    });
})();