﻿(function () {

    var filename = "userdata.txt";

    var currentUserFileVersion = function () {
        var current = Windows.Storage.ApplicationData.current.localSettings.values["userFileVersion"];

        if (current)
            return parseInt(current);

        return 0;
    };

    var updateDistantFileVersion = function (version) {
        var fileID = Windows.Storage.ApplicationData.current.roamingSettings.values["fileID"];

        if (!fileID)
            return;

        WL.api({
            path: fileID,
            method: "PUT",
            body: {
                description: version
            }
        }).done();
    };

    var connect = function () {
        if (UrzaGatherer.Skydrive.OnConnecting)
            UrzaGatherer.Skydrive.OnConnecting();

        WL.login({
            scope: ["wl.signin", "wl.skydrive_update"]
        }).then(function (response) {
            getUserFile();
        }, function (responseFailed) {

            if (responseFailed.error == "access_denied") {
            }
            else {
                if (UrzaGatherer.Skydrive.OnFailed)
                    UrzaGatherer.Skydrive.OnFailed();
            }
        });
    };

    var uploadUserFile = function (deferral) {
        Windows.Storage.ApplicationData.current.localFolder.createFileAsync(filename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
            Windows.Storage.FileIO.writeTextAsync(file, JSON.stringify(UrzaGatherer.UserData)).then(function () {
                var folderID = Windows.Storage.ApplicationData.current.roamingSettings.values["folderID"];
                Windows.Storage.ApplicationData.current.localSettings.values["userFileVersion"] = currentUserFileVersion() + 1;

                WL.upload({
                    path: folderID,
                    file_name: file.name,
                    file_input: file,
                    overwrite: true
                }).then(
                function (response) {
                    Windows.Storage.ApplicationData.current.roamingSettings.values["fileID"] = response.id;
                    updateDistantFileVersion(currentUserFileVersion());

                    if (deferral)
                        deferral.complete();
                },
                function (responseFailed) {
                    if (deferral)
                        deferral.complete();
                }
            );
            });
        });
    };

    var getFileOnline = function () {
        var fileID = Windows.Storage.ApplicationData.current.roamingSettings.values["fileID"];

        if (!fileID) {
            getFileOffline();
            return;
        }

        // Get file info
        WL.api({
            path: fileID,
            method: "GET"
        }).then(
                function (response) {
                    var distantVersion = parseInt(response.description ? response.description : "-1");
                    var localVersion = currentUserFileVersion();

                    // Download file
                    if (localVersion < distantVersion) {
                        Windows.Storage.ApplicationData.current.localFolder.createFileAsync(filename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                            WL.download({
                                path: fileID,
                                file_output: file
                            }).then(
                            function (response) {
                                getFileOffline();
                            },
                            function (responseFailed) {
                                getFileOffline();
                            }
                        );
                        });
                        return;
                    }
                    getFileOffline();
                },
                function (responseFailed) {
                    getFileOffline();
                }
            );
    };

    var getFileOffline = function () {
        Windows.Storage.ApplicationData.current.localFolder.getFileAsync(filename).then(function (file) {
            // On success
            Windows.Storage.FileIO.readTextAsync(file).then(function (data) {
                if (data)
                    UrzaGatherer.UserData = JSON.parse(data);
                else
                    UrzaGatherer.UserData = {};

                if (UrzaGatherer.Skydrive.OnConnected && UrzaGatherer.UserLogged)
                    UrzaGatherer.Skydrive.OnConnected();

                if (UrzaGatherer.Skydrive.OnDataAvailable)
                    UrzaGatherer.Skydrive.OnDataAvailable();
            });
        }, function () {
            // On error
            Windows.Storage.ApplicationData.current.localFolder.createFileAsync(filename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                UrzaGatherer.UserData = {};
                Windows.Storage.FileIO.writeTextAsync(file, JSON.stringify(UrzaGatherer.UserData));
                Windows.Storage.ApplicationData.current.localSettings.values["userFileVersion"] = currentUserFileVersion() + 1;

                if (UrzaGatherer.Skydrive.OnConnected && UrzaGatherer.UserLogged)
                    UrzaGatherer.Skydrive.OnConnected();

                if (UrzaGatherer.Skydrive.OnDataAvailable)
                    UrzaGatherer.Skydrive.OnDataAvailable();
            });
        });
    };

    var getUserFile = function () {
        UrzaGatherer.UserLogged = true;

        // Creating folder
        WL.api({
            path: "me/skydrive",
            method: "POST",
            body: {
                "name": "UrzaGatherer",
                "description": "UrzaGatherer repository folder"
            }
        }).then(
                function (response) {
                    Windows.Storage.ApplicationData.current.roamingSettings.values["folderID"] = response.id;
                    getFileOnline();
                },
                function (responseFailed) {
                    if (responseFailed.error.code != "resource_already_exists") {
                        getFileOffline();
                        if (UrzaGatherer.Skydrive.OnFailed)
                            UrzaGatherer.Skydrive.OnFailed();
                    }
                    else {
                        getFileOnline();
                    }
                }
            );

    };

    var init = function () {
        var onStatusChange = function (e) {
            WL.getLoginStatus(function (response) {
                if (response.status == "notConnected") {
                    connect();
                }
            });
        };

        WL.Event.subscribe("auth.login", getUserFile);
        WL.Event.subscribe("auth.statusChange", onStatusChange);

        WL.init({
            scope: ["wl.signin", "wl.skydrive_update"]
        });
    };

    WinJS.Namespace.define("UrzaGatherer.Skydrive", {
        Init: init,
        Connect: connect,
        UploadUserFile: uploadUserFile
    });
})();