﻿(function () {
    "use strict";

    var delayImageLoader = WinJS.Class.define(
            function (element, options) {
                this._element = element || document.createElement("div");
                this.element.winControl = this;
                var progress = document.createElement("progress");

                var downloadImage = function (source, filename, img) {
                    var url = options ? UrzaGatherer.Root + "/" + options.root + "/" + source : UrzaGatherer.Root + "/" + source;
                    var xmlRequest = new XMLHttpRequest();
                    xmlRequest.open("GET", url, true);
                    xmlRequest.responseType = "blob";
                    xmlRequest.onreadystatechange = function () {
                        if (xmlRequest.readyState === 4) {
                            WinJS.Utilities.addClass(progress, "hidden");
                            UrzaGatherer.Stats.DecrementDownloads();
                            if (xmlRequest.status == 200) {
                                var blob = xmlRequest.response;
                                var input = blob.msDetachStream();

                                Windows.Storage.ApplicationData.current.localFolder.createFileAsync(filename, Windows.Storage.CreationCollisionOption.replaceExisting).then(function (file) {
                                    file.openAsync(Windows.Storage.FileAccessMode.readWrite).then(function (output) {
                                        Windows.Storage.Streams.RandomAccessStream.copyAsync(input, output).then(function () {
                                            output.flushAsync().then(function () {
                                                input.close();
                                                output.close();
                                                // Loading using web cache
                                                img.src = options ? UrzaGatherer.Root + "/" + options.root + "/" + source : UrzaGatherer.Root + "/" + source;
                                            });
                                        });
                                    });
                                });
                            }
                        }
                    };

                    UrzaGatherer.Stats.IncrementDownloads();
                    xmlRequest.send(null);

                }

                WinJS.Utilities.addClass(this.element, "imageLoaderRoot");

                progress.className = "win-ring";

                this.element.appendChild(progress);

                WinJS.Utilities.query("img", element).forEach(function (img) {
                    WinJS.Utilities.addClass(img, "imageLoader");

                    img.addEventListener("load", function () {
                        WinJS.Utilities.addClass(img, "loaded");
                        WinJS.Utilities.addClass(progress, "hidden");
                    });

                    img.addEventListener("error", function (err) {
                        if (img.src.substring(0, 5) == "http:" || img.src.indexOf("hideBanner") != -1) {
                            WinJS.Utilities.addClass(progress, "hidden");
                        }
                        else {
                            var source = img.src.replace("ms-appdata:///local/", "");
                            var filename = source.replace("/", "\\");

                            Windows.Storage.ApplicationData.current.localFolder.getFileAsync(filename).then(function (file) {
                                var url = options ? UrzaGatherer.Root + "/" + options.root + "/" + source : UrzaGatherer.Root + "/" + source;
                                img.src = url;

                            }, function () { // Not found
                                downloadImage(source, filename, img);
                            });
                        }
                    });
                });
            },
            {
                element: {
                    get: function () { return this._element; }
                },
            });

    var findCSS = function (name) {
        for (var index = 0; index < document.styleSheets.length; index++) {
            if (document.styleSheets[index].href.indexOf(name) != -1)
                return document.styleSheets[index];
        }
    }

    var getCardFile = function (card) {
        // HTTP ?
        if (card.logo.substring(0, 5).toLowerCase() == "http:") {
            var uri = new Windows.Foundation.Uri(card.logo);
            var thumbnail = Windows.Storage.Streams.RandomAccessStreamReference.createFromUri(uri);

            return Windows.Storage.StorageFile.createStreamedFileFromUriAsync(card.name + ".jpg", uri, thumbnail);
        }

        // Local ?
        return Windows.Storage.ApplicationData.current.localFolder.getFileAsync(card.alt);
    }

    var updateTile = function (card) {
        var Notifications = Windows.UI.Notifications;
        var Imaging = Windows.Graphics.Imaging;

        var tileXml = Notifications.TileUpdateManager.getTemplateContent(Notifications.TileTemplateType.tileWidePeekImage02);

        var tileTextAttributes = tileXml.getElementsByTagName("text");
        tileTextAttributes[0].appendChild(tileXml.createTextNode("UrzaGatherer"));
        tileTextAttributes[1].appendChild(tileXml.createTextNode(card.name));
        tileTextAttributes[2].appendChild(tileXml.createTextNode(card.expansion.name + " / " + card.expansion.block.name));
        tileTextAttributes[3].appendChild(tileXml.createTextNode(UrzaGatherer.Tools.GetString("Author") + " " + card.author));

        if (!card.alt)
            return;

        var filename = card.alt.replace(".jpg", "_small.png");
        UrzaGatherer.Imaging.RescaleAndCropImage(card, 310, 150, filename, true, function (appDatafilename) {
            // Image
            var tileImageAttributes = tileXml.getElementsByTagName("image");
            tileImageAttributes[0].setAttribute("src", appDatafilename);

            // Square
            var squareTileXml = Notifications.TileUpdateManager.getTemplateContent(Notifications.TileTemplateType.tileSquareImage);
            var squareTileImageAttributes = squareTileXml.getElementsByTagName("image");
            squareTileImageAttributes[0].setAttribute("src", appDatafilename);

            var node = tileXml.importNode(squareTileXml.getElementsByTagName("binding").item(0), true);
            tileXml.getElementsByTagName("visual").item(0).appendChild(node);

            // Update
            var tileNotification = new Notifications.TileNotification(tileXml);
            tileNotification.tag = card.id;

            var tileUpdater = Windows.UI.Notifications.TileUpdateManager.createTileUpdaterForApplication();

            tileUpdater.enableNotificationQueue(true);
            tileUpdater.update(tileNotification);

            // Store and clean
            var previousTilesValue = Windows.Storage.ApplicationData.current.localSettings.values["previousTiles"];
            var previousTiles = [];

            if (previousTilesValue)
                previousTiles = JSON.parse(previousTilesValue);

            previousTiles.push(filename);

            if (previousTiles.length > 5) {
                var toRemove = previousTiles.shift();

                Windows.Storage.ApplicationData.current.localFolder.getFileAsync(toRemove).then(function (file) {
                    file.deleteAsync().done();
                });
            }

            Windows.Storage.ApplicationData.current.localSettings.values["previousTiles"] = JSON.stringify(previousTiles);
        });

    }

    var showWarning = function (key) {
        var msg = new Windows.UI.Popups.MessageDialog(UrzaGatherer.Tools.GetString(key), "UrzaGatherer");

        msg.showAsync();
    };

    var resourceLoader = new Windows.ApplicationModel.Resources.ResourceLoader();

    var priceConverter = WinJS.Binding.converter(function (value) {
        if (!value) {
            return UrzaGatherer.Tools.GetString("Unknown");
        }
        return Math.round(value * 100) / 100 + "€";
    });

    var cardsCountConverter = WinJS.Binding.converter(function (value) {
        return value + UrzaGatherer.Tools.GetString("Cards");
    });

    var boolToStringConverter = WinJS.Binding.converter(function (value) {
        return value ? UrzaGatherer.Tools.GetString("Yes") : UrzaGatherer.Tools.GetString("No");
    });

    var boolToColorConverter = WinJS.Binding.converter(function (value) {
        return value ? "#4617B4" : "black";
    });

    var boolToDisplayConverter = WinJS.Binding.converter(function (value) {
        return value ? "inherit" : "none";
    });

    var boolToInvertDisplayConverter = WinJS.Binding.converter(function (value) {
        return value ? "none" : "inherit";
    });

    var textToMultilineConverter = WinJS.Binding.converter(function (value) {
        return value.replace(/\r\n/g, "<br>");
    });

    var textConverter = WinJS.Binding.converter(function (value) {
        if (!value)
            return "N/A";

        var result = value.replace(/\r\n/g, "<br>");

        // Phyrexia
        result = result.replace(new RegExp("\\(RP\\)", "g"), "<img class='symbol' src='/images/symbols/rp.png'>");
        result = result.replace(new RegExp("\\(GP\\)", "g"), "<img class='symbol' src='/images/symbols/gp.png'>");
        result = result.replace(new RegExp("\\(UP\\)", "g"), "<img class='symbol' src='/images/symbols/up.png'>");
        result = result.replace(new RegExp("\\(WP\\)", "g"), "<img class='symbol' src='/images/symbols/wp.png'>");
        result = result.replace(new RegExp("\\(BP\\)", "g"), "<img class='symbol' src='/images/symbols/bp.png'>");

        // Replace
        for (var index = 0; index < 17; index++) {
            result = result.replace(new RegExp("\\(" + index + "\\)", "g"), "<img class='symbol' src='/images/symbols/" + index + ".png'>");
        }
        result = result.replace(new RegExp("\\(T\\)", "g"), "<img class='symbol' src='/images/symbols/T.png'>");
        result = result.replace(new RegExp("\\(-T\\)", "g"), "<img class='symbol' src='/images/symbols/untap.png'>");
        result = result.replace(new RegExp("\\(X\\)", "g"), "<img class='symbol' src='/images/symbols/x.png'>");

        result = result.replace(new RegExp("\\(R\\)", "g"), "<img class='symbol' src='/images/symbols/red.png'>");
        result = result.replace(new RegExp("\\(G\\)", "g"), "<img class='symbol' src='/images/symbols/green.png'>");
        result = result.replace(new RegExp("\\(U\\)", "g"), "<img class='symbol' src='/images/symbols/blue.png'>");
        result = result.replace(new RegExp("\\(W\\)", "g"), "<img class='symbol' src='/images/symbols/white.png'>");
        result = result.replace(new RegExp("\\(B\\)", "g"), "<img class='symbol' src='/images/symbols/black.png'>");

        result = result.replace(new RegExp("\\(2R\\)", "g"), "<img class='symbol' src='/images/symbols/2r.png'>");
        result = result.replace(new RegExp("\\(2G\\)", "g"), "<img class='symbol' src='/images/symbols/2g.png'>");
        result = result.replace(new RegExp("\\(2U\\)", "g"), "<img class='symbol' src='/images/symbols/2u.png'>");
        result = result.replace(new RegExp("\\(2W\\)", "g"), "<img class='symbol' src='/images/symbols/2w.png'>");
        result = result.replace(new RegExp("\\(2B\\)", "g"), "<img class='symbol' src='/images/symbols/2b.png'>");

        result = result.replace(new RegExp("\\(RG\\)", "g"), "<img class='symbol' src='/images/symbols/RG.png'>");
        result = result.replace(new RegExp("\\(RW\\)", "g"), "<img class='symbol' src='/images/symbols/RW.png'>");
        result = result.replace(new RegExp("\\(BG\\)", "g"), "<img class='symbol' src='/images/symbols/BG.png'>");
        result = result.replace(new RegExp("\\(BR\\)", "g"), "<img class='symbol' src='/images/symbols/BR.png'>");
        result = result.replace(new RegExp("\\(UR\\)", "g"), "<img class='symbol' src='/images/symbols/UR.png'>");
        result = result.replace(new RegExp("\\(UB\\)", "g"), "<img class='symbol' src='/images/symbols/UB.png'>");
        result = result.replace(new RegExp("\\(WB\\)", "g"), "<img class='symbol' src='/images/symbols/WB.png'>");
        result = result.replace(new RegExp("\\(WU\\)", "g"), "<img class='symbol' src='/images/symbols/WU.png'>");
        result = result.replace(new RegExp("\\(GU\\)", "g"), "<img class='symbol' src='/images/symbols/GU.png'>");
        result = result.replace(new RegExp("\\(GW\\)", "g"), "<img class='symbol' src='/images/symbols/GW.png'>");

        result = result.replace(new RegExp("\\(100\\)", "g"), "<img class='symbol' src='/images/symbols/100.png'>");
        result = result.replace(new RegExp("\\(500\\)", "g"), "<img class='symbol' src='/images/symbols/500.png'>");
        result = result.replace(new RegExp("\\(Infinity\\)", "g"), "<img class='symbol' src='/images/symbols/Infinity.png'>");

        return result;
    });

    var rarityToColorConverter = WinJS.Binding.converter(function (value) {
        switch (value) {
            case 4:
                return "#FA8824";
            case 3:
                return "Gold";
            case 2:
                return "#657177";
            case 1:
                return "Black";
        }
    });

    var colorConverter = WinJS.Binding.converter(function (value) {
        switch (value) {
            case 1:
                return "Black";
            case 2:
                return "Red";
            case 3:
                return "Blue";
            case 4:
                return "White";
            case 5:
                return "Green";
            case 6:
                return "gray";
            case 7:
                return "#816849";
            case 8:
                return "gold";
            case 9:
                return "#8888cc";
        }

        return value;
    });

    var largeToMaxHeightConverter = WinJS.Binding.converter(function (value) {
        if (value.isPlane) {
            if (window.innerHeight < 1024) {
                if (window.innerWidth <= 1024)
                    return "200px";
                return "510px";
            }
        }

        if (value.isScheme && window.innerHeight > 1024)
            return "900px";

        if (window.innerHeight < 1024)
            return "630px";

        return "680px";
    });

    var largeToWidthConverter = WinJS.Binding.converter(function (value) {
        return value.isPlane ? "240px" : "120px";
    });

    var authorToHrefConverter = WinJS.Binding.converter(function (value) {
        return "/pages/search/search.html";
    });

    WinJS.Namespace.define("UrzaGatherer.Tools", {
        DelayImageLoader: delayImageLoader,
        FindCSS: findCSS,
        GetString: function (name) {
            if (!name)
                return "";

            return resourceLoader.getString(name);
        },
        PriceConverter: priceConverter,
        CardsCountConverter: cardsCountConverter,
        BoolToStringConverter: boolToStringConverter,
        BoolToColorConverter: boolToColorConverter,
        RarityToColorConverter: rarityToColorConverter,
        BoolToDisplayConverter: boolToDisplayConverter,
        BoolToInvertDisplayConverter: boolToInvertDisplayConverter,
        AuthorToHrefConverter: authorToHrefConverter,
        TextToMultilineConverter: textToMultilineConverter,
        LargeToMaxHeightConverter: largeToMaxHeightConverter,
        LargeToWidthConverter: largeToWidthConverter,
        TextConverter: textConverter,
        ColorConverter: colorConverter,
        GetCardFile: getCardFile,
        UpdateTile: updateTile,
        ShowWarning: showWarning
    });

    WinJS.Namespace.define("UrzaGatherer.Store", {
    });
})();