﻿(function () {
    "use strict";

    var appView = Windows.UI.ViewManagement.ApplicationView;
    var appViewState = Windows.UI.ViewManagement.ApplicationViewState;
    var nav = WinJS.Navigation;
    var ui = WinJS.UI;
    var utils = WinJS.Utilities;
    var favoriteButton;

    var card;
    var globalOptions;

    var shareFunction = function (e) {
        var request = e.request;
        var deferral = request.getDeferral();

        request.data.properties.title = card.name + UrzaGatherer.Tools.GetString("ShareTitle") + card.expansion.name;
        request.data.properties.description = UrzaGatherer.Tools.GetString("ShareDescription");

        UrzaGatherer.Tools.GetCardFile(card).then(function (file) {
            var streamReference = Windows.Storage.Streams.RandomAccessStreamReference.createFromFile(file);
            request.data.properties.thumbnail = streamReference;

            request.data.setStorageItems([file]);

            request.data.setBitmap(streamReference);

            deferral.complete();
        });
    };

    var processAppBar = function () {

        if (card && UrzaGatherer.UserData && UrzaGatherer.UserData.favorites && UrzaGatherer.UserData.favorites.indexOf(card.id) != -1) {
            favoriteButton.winControl.label = UrzaGatherer.Tools.GetString("RemoveFavorite");
            favoriteButton.winControl.icon = WinJS.UI.AppBarIcon.unfavorite;
            favoriteButton.winControl.tooltip = UrzaGatherer.Tools.GetString("RemoveFavoriteTooltip");
        }
        else {
            favoriteButton.winControl.label = UrzaGatherer.Tools.GetString("AddFavorite");
            favoriteButton.winControl.icon = WinJS.UI.AppBarIcon.favorite;
            favoriteButton.winControl.tooltip = UrzaGatherer.Tools.GetString("AddFavoriteTooltip");
        }
    };

    var shouldRestoreLegend;

    ui.Pages.define("/pages/card/card.html", {
        unload: function () {
            var dataTransferManager = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
            dataTransferManager.removeEventListener("datarequested", shareFunction);

            if (shouldRestoreLegend)
                WinJS.Utilities.removeClass(document.querySelector(".legend"), "hidden");

            var bar = document.querySelector(".appBar");

            bar.parentNode.removeChild(bar);
        },
        ready: function (element, options) {
            WinJS.Resources.processAll();

            globalOptions = options;

            var legend = document.querySelector(".legend");

            if (!WinJS.Utilities.hasClass(legend, "hidden")) {
                WinJS.Utilities.addClass(legend, "hidden");
                shouldRestoreLegend = true;
            }
            else
                shouldRestoreLegend = false;

            favoriteButton = element.querySelector("#favoriteButton");

            // Share
            var dataTransferManager = Windows.ApplicationModel.DataTransfer.DataTransferManager.getForCurrentView();
            dataTransferManager.addEventListener("datarequested", shareFunction);

            // Flip view
            var flipView = document.getElementById("flipView").winControl;
            var currentPage = -1;
            var background = document.querySelector("#background");

            flipView.addEventListener("pageselected", function (evt) {
                options.cardIndex = flipView.currentPage;

                // Back
                var backButtons = evt.detail.source.querySelectorAll(".win-backbutton");
                for (var index = 0; index < backButtons.length; index++) {
                    var backButton = backButtons[index];
                    backButton.onclick = function () { nav.back(); };
                }

                // Author
                var authorButtons = evt.detail.source.querySelectorAll(".item-author");
                for (var index = 0; index < authorButtons.length; index++) {
                    var authorButton = authorButtons[index];
                    authorButton.onclick = function () {
                        nav.navigate("/pages/search/search.html", { authorQuery: options.cards.getAt(flipView.currentPage).author.toLowerCase() });
                    };
                }

                // Rarity
                var rarityButtons = evt.detail.source.querySelectorAll(".item-rarity");
                for (var index = 0; index < rarityButtons.length; index++) {
                    var rarityButton = rarityButtons[index];
                    rarityButton.onclick = function () {
                        nav.navigate("/pages/search/search.html", { rarityQuery: options.cards.getAt(flipView.currentPage).rarity.toLowerCase() });
                    };
                }

                // Color
                var colorButtons = evt.detail.source.querySelectorAll(".item-color");
                for (var index = 0; index < colorButtons.length; index++) {
                    var colorButton = colorButtons[index];
                    colorButton.onclick = function () {
                        nav.navigate("/pages/search/search.html", { colorQuery: options.cards.getAt(flipView.currentPage).color.toLowerCase() });
                    };
                }

                // Expansion
                var expansionButtons = evt.detail.source.querySelectorAll(".expansion-image-container");
                for (var index = 0; index < expansionButtons.length; index++) {
                    var expansionButton = expansionButtons[index];
                    expansionButton.onclick = function () {
                        nav.navigate("/pages/expansion/expansion.html", { expansion: options.cards.getAt(flipView.currentPage).expansion });
                    };
                }

                if (currentPage == flipView.currentPage)
                    return;

                currentPage = flipView.currentPage;
                card = options.cards.getAt(flipView.currentPage);

                // Check buttons
                var checkButton = element.querySelector("#checkButton");
                var uncheckButton = element.querySelector("#uncheckButton");

                if (card.isChecked) {
                    if (!WinJS.Utilities.hasClass(checkButton, "hidden"))
                        WinJS.Utilities.addClass(checkButton, "hidden");

                    if (WinJS.Utilities.hasClass(uncheckButton, "hidden"))
                        WinJS.Utilities.removeClass(uncheckButton, "hidden");
                } else {
                    if (!WinJS.Utilities.hasClass(uncheckButton, "hidden"))
                        WinJS.Utilities.addClass(uncheckButton, "hidden");

                    if (WinJS.Utilities.hasClass(checkButton, "hidden"))
                        WinJS.Utilities.removeClass(checkButton, "hidden");
                }

                if (card.alt) {
                    UrzaGatherer.Tools.UpdateTile(card);

                    setImmediate(function () {
                        var dynamicBackground = Windows.Storage.ApplicationData.current.roamingSettings.values["dynamicBackground"];
                        if (dynamicBackground) {
                            UrzaGatherer.Imaging.CropAndDarkenImage(card, 0.8, function (uri) {
                                var otherBackground;

                                if (background.id == "background") {
                                    otherBackground = document.querySelector("#background2");
                                }
                                else {
                                    otherBackground = document.querySelector("#background");
                                }

                                if (otherBackground == null)
                                    return;

                                otherBackground.style.backgroundImage = "url('" + uri + "')";
                                otherBackground.style.opacity = 0.9999; // Bug?
                                background.style.opacity = 0;

                                background = otherBackground;
                            });
                        }
                    });
                }

                // Appbar
                processAppBar();
            });

            ui.setOptions(flipView, {
                itemDataSource: options.cards.dataSource,
                currentPage: options.cardIndex
            });

            this.updateLayout(element, appView.value);

            // onDataStatusChanged
            UrzaGatherer.Collection.onDataStatusChanged = function () {
                processAppBar();
            }

            // Appbar
            favoriteButton.onclick = function () {
                if (!UrzaGatherer.UserData) {
                    UrzaGatherer.Tools.ShowWarning("UserDataNotYetLoaded");
                    return;
                }
                UrzaGatherer.UserData.favorites = UrzaGatherer.UserData.favorites || [];

                var index = UrzaGatherer.UserData.favorites.indexOf(card.id);
                if (index != -1) {
                    UrzaGatherer.UserData.favorites.splice(index, 1);
                    UrzaGatherer.FavoritesList.splice(index, 1);
                }
                else {
                    UrzaGatherer.UserData.favorites.push(card.id);
                    UrzaGatherer.FavoritesList.push(card);
                }

                UrzaGatherer.Collection.SaveFile();

                processAppBar();
            };

            // Gatherer
            var gathererButton = element.querySelector("#gathererButton");
            gathererButton.winControl.label = UrzaGatherer.Tools.GetString("Gatherer");
            gathererButton.winControl.tooltip = UrzaGatherer.Tools.GetString("GathererTooltip");

            gathererButton.onclick = function () {
                Windows.System.Launcher.launchUriAsync(new Windows.Foundation.Uri("http://gatherer.wizards.com/Pages/Card/Details.aspx?multiverseid=" + card.multiverseId.toString()));
            }

            // Buy
            var buyButton = element.querySelector("#buyButton");
            buyButton.winControl.label = UrzaGatherer.Tools.GetString("Buy");
            buyButton.winControl.tooltip = UrzaGatherer.Tools.GetString("BuyTooltip");

            buyButton.onclick = function () {
                var name = card.nameEn;

                for (var index = 1; index < 5; index++) {
                    name = name.replace("(v. " + index.toString() + ")", "");
                    name = name.replace("(v." + index.toString() + ")", "");
                }
                var uri;

                if (card.cardMarketLink)
                    uri = new Windows.Foundation.Uri(card.cardMarketLink);
                else
                    uri = new Windows.Foundation.Uri("https://www.magiccardmarket.eu/?mainPage=showSearchResult&searchFor=" + name);

                Windows.System.Launcher.launchUriAsync(uri);
            }

            // Selection
            var checkButton = element.querySelector("#checkButton");
            var uncheckButton = element.querySelector("#uncheckButton");

            checkButton.winControl.label = UrzaGatherer.Tools.GetString("CheckLabel");
            checkButton.winControl.icon = WinJS.UI.AppBarIcon.accept;
            checkButton.winControl.tooltip = UrzaGatherer.Tools.GetString("CheckTooltip");
            uncheckButton.winControl.label = UrzaGatherer.Tools.GetString("UncheckLabel");
            uncheckButton.winControl.icon = WinJS.UI.AppBarIcon.clear;
            uncheckButton.winControl.tooltip = UrzaGatherer.Tools.GetString("UncheckTooltip");

            checkButton.onclick = function (e) {
                card.isChecked = true;
                UrzaGatherer.UserData[card.id] = true;pai
                card.expansion.incrementCompletion();

                UrzaGatherer.Collection.SaveFile();

                WinJS.Utilities.addClass(checkButton, "hidden");
                WinJS.Utilities.removeClass(uncheckButton, "hidden");
            };

            uncheckButton.onclick = function (e) {
                card.isChecked = false;
                UrzaGatherer.UserData[card.id] = false;
                card.expansion.decrementCompletion();

                UrzaGatherer.Collection.SaveFile();

                WinJS.Utilities.addClass(uncheckButton, "hidden");
                WinJS.Utilities.removeClass(checkButton, "hidden");

            };
        },

        updateLayout: function (element, viewState, previousViewState) {
            if (previousViewState == viewState)
                return;

            var flipView = document.getElementById("flipView").winControl;

            if (viewState === appViewState.snapped) {
            } else {
                ui.setOptions(flipView, {
                    currentPage: globalOptions.cardIndex
                });

            }
        }

    });
})();
