﻿(function () {
    "use strict";

    var appView = Windows.UI.ViewManagement.ApplicationView;
    var appViewState = Windows.UI.ViewManagement.ApplicationViewState;
    var nav = WinJS.Navigation;
    var ui = WinJS.UI;
    var utils = WinJS.Utilities;

    var expansion;
    var filters = [["color", UrzaGatherer.Tools.GetString("Colors")], ["author", UrzaGatherer.Tools.GetString("Authors")], ["rarity", UrzaGatherer.Tools.GetString("Rarities")], ["type", UrzaGatherer.Tools.GetString("Types")]];
    var filtered;
    var all = UrzaGatherer.Tools.GetString("All");

    // Filters
    var currentAdvancedFiltersState;

    var currentFilters;
    var currentTextFilter;
    var textFilterOption;
    var currentCheckFilter;

    var globalOptions;

    var getStats = function () {
        var stats = UrzaGatherer.GetStatistics(filtered);
        var cardsCount = document.getElementById("cardsCount");

        cardsCount.innerText = stats.cards;
        document.getElementById("cardsPrice").innerText = stats.price;

        if (expansion.isChecked) {
            WinJS.Utilities.addClass(cardsCount, "complete");
        }
        else {
            WinJS.Utilities.removeClass(cardsCount, "complete");
        }
    }

    var updateSize = function (forceUpdate) {
        var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];

        if (!zoomLevel)
            return;

        var css = UrzaGatherer.Tools.FindCSS("expansion.css");
        var level = zoomLevel / 100.0;

        for (var index = 0; index < css.cssRules.length; index++) {
            if (css.cssRules[index].selectorText == ".expansion .cardsList .item-container") {
                css.cssRules[index].style.width = (480 * level) + "px";
                css.cssRules[index].style.height = (680 * level) + "px";
            }
        }

        if (forceUpdate) {
            var listView = document.querySelector(".cardsList").winControl;
            listView.forceLayout();
        }
    }

    var filterFunction = function (item) {
        //Filters
        for (var index = 0; index < filters.length; index++) {
            var filter = currentFilters[index];

            if (filter.substring(0, all.length) != all) {
                if (item[filters[index][0]] != filter)
                    return false;
            }
        }

        // Text
        if (!UrzaGatherer.Common.CheckTextFilter(currentTextFilter, item))
            return false;

        // Check
        if (UrzaGatherer.UserData) {
            if (currentCheckFilter > 0) {
                if (!((currentCheckFilter == 1 && !item.isChecked) || (currentCheckFilter == 2 && item.isChecked)))
                    return false;
            }
        }

        // Advanced filters
        if (currentAdvancedFiltersState.manaCostFilterActivated) {
            if (!(item.convertedManaCost >= currentAdvancedFiltersState.currentMinManaCost && item.convertedManaCost <= currentAdvancedFiltersState.currentMaxManaCost))
                return false;
        }

        if (currentAdvancedFiltersState.priceFilterActivated) {
            if (!(item.price !== undefined && item.price >= currentAdvancedFiltersState.currentMinPrice && item.price <= currentAdvancedFiltersState.currentMaxPrice))
                return false;
        }

        if (currentAdvancedFiltersState.forceFilterActivated) {
            if (!(item.force !== undefined && item.force >= currentAdvancedFiltersState.currentMinForce && item.force <= currentAdvancedFiltersState.currentMaxForce))
                return false;
        }

        if (currentAdvancedFiltersState.defenseFilterActivated) {
            if (!(item.defense !== undefined && item.defense >= currentAdvancedFiltersState.currentMinDefense && item.defense <= currentAdvancedFiltersState.currentMaxDefense))
                return false;
        }

        return true;
    }

    var prepareFilter = function (property, plural, that) {
        var filter = document.getElementById(property + "Filter");
        var results = [];

        for (var cardIndex = 0; cardIndex < expansion.cards.length; cardIndex++) {
            var value = expansion.cards[cardIndex][property];

            if (results.indexOf(value) == -1)
                results.push(value);
        }
        var allLength = all.length;

        results.push(all + plural);
        var sortedResults = results.sort(function (i0, i1) {
            if (i0 == i1)
                return 0;

            if (i0.substring(0, allLength) == all)
                return -1;

            if (i1.substring(0, allLength) == all)
                return 1;

            if (i0 > i1)
                return 1;

            return -1;
        });

        for (var index = 0; index < sortedResults.length; index++) {
            filter.add(new Option(sortedResults[index]));

            var stored = globalOptions["expansion" + property + "Filter"];

            if (stored && stored == sortedResults[index]) {
                filter.selectedIndex = index;
                delete globalOptions["expansion" + property + "Filter"];
            }
        }

        filter.addEventListener("change", function () {
            that.updateLayout(document, appView.value);
        });
    };

    var pinByElementAsync = function (then) {
        var tileID = expansion.id;
        var shortName = "UrzaGatherer - " + expansion.name;
        var displayName = "UrzaGatherer - " + expansion.name;
        var tileOptions = Windows.UI.StartScreen.TileOptions.showNameOnWideLogo;
        var tileActivationArguments = expansion.id;

        UrzaGatherer.Imaging.RescaleImage(expansion.logo, 150, 150, expansion.logoPath.replace(".png", "_uri.png"), false, function (uriLogo) {
            var tile = new Windows.UI.StartScreen.SecondaryTile(tileID, shortName, displayName, tileActivationArguments, tileOptions, new Windows.Foundation.Uri(uriLogo));
            tile.foregroundText = Windows.UI.StartScreen.ForegroundText.dark;

            UrzaGatherer.Imaging.RescaleImage(expansion.logo, 310, 150, expansion.logoPath.replace(".png", "_wide.png"), false, function (wideLogo) {
                tile.wideLogo = new Windows.Foundation.Uri(wideLogo);

                var element = document.getElementById("pinButton");
                var selectionRect = element.getBoundingClientRect();

                tile.requestCreateAsync({ x: selectionRect.left, y: selectionRect.top }).then(function (isCreated) {
                    if (isCreated)
                        then();
                });
            });
        });
    };

    var removeFromCollection = function (that, tab) {
        if (!UrzaGatherer.UserData) {
            UrzaGatherer.Tools.ShowWarning("UserDataNotYetLoaded");
            return;
        }

        for (var index = 0; index < (tab ? tab.length : filtered.length) ; index++) {
            var card = filtered.getAt(tab ? tab[index] : index);

            if (card.isChecked) {
                card.isChecked = false;
                card.expansion.decrementCompletion();
            }

            delete UrzaGatherer.UserData[card.id];
        };

        var listView = document.querySelector(".cardsList").winControl;
        listView.selection.clear();

        var currentIndex = listView.indexOfFirstVisible;

        if (document.getElementById("checkFilter").selectedIndex > 0)
            that.updateLayout(document, appView.value);

        UrzaGatherer.Collection.SaveFile();

        document.querySelector(".appBar").winControl.hide();

        getStats();

        if (currentIndex)
            listView.indexOfFirstVisible = currentIndex;
    };

    var addToCollection = function (that, tab) {
        if (!UrzaGatherer.UserData) {
            UrzaGatherer.Tools.ShowWarning("UserDataNotYetLoaded");
            return;
        }

        for (var index = 0; index < (tab ? tab.length : filtered.length) ; index++) {
            var card = filtered.getAt(tab ? tab[index] : index);

            if (!card.isChecked) {
                card.isChecked = true;
                card.expansion.incrementCompletion();
            }

            UrzaGatherer.UserData[card.id] = true;
        };

        var listView = document.querySelector(".cardsList").winControl;
        listView.selection.clear();

        var currentIndex = listView.indexOfFirstVisible;

        if (document.getElementById("checkFilter").selectedIndex > 0)
            that.updateLayout(document, appView.value);

        UrzaGatherer.Collection.SaveFile();

        document.querySelector(".appBar").winControl.hide();

        getStats();

        setImmediate(function () {
            if (currentIndex)
                listView.indexOfFirstVisible = currentIndex;
        });
    };

    var saveState = function () {
        // Saving state
        for (var index = 0; index < filters.length; index++) {
            globalOptions["expansion" + filters[index][0] + "Filter"] = document.getElementById(filters[index][0] + "Filter").value;
        }

        globalOptions.expansionOrderSelect = document.getElementById("orderSelect").selectedIndex;
        globalOptions.expansionTextFilter = document.getElementById("textFilter").value;
        globalOptions.expansionCheckFilter = document.getElementById("checkFilter").selectedIndex;

        globalOptions.expansionScrollPosition = document.querySelector(".cardsList").winControl.indexOfFirstVisible;
    }

    var onDataChanged = function () {
        updateSize(true);
    }

    ui.Pages.define("/pages/expansion/expansion.html", {
        unload: function () {
            var bar = document.querySelector(".appBar");

            bar.parentNode.removeChild(bar);

            nav.removeEventListener("navigating", saveState);

            Windows.Storage.ApplicationData.current.removeEventListener("datachanged", onDataChanged);
        },
        afterPageEnter: function (element) {
            nav.addEventListener("navigating", saveState);
            var that = this;

            // List view
            var listView = element.querySelector(".cardsList").winControl;

            if (UrzaGatherer.UserData) {
                listView.selectionMode = WinJS.UI.SelectionMode.multi;
                WinJS.Utilities.removeClass(document.querySelector("#checkFilter"), "hidden");
            }

            ui.setOptions(listView, {
                itemTemplate: UrzaGatherer.Common.ItemRenderer,
                oniteminvoked: this.itemInvoked
            });

            listView.addEventListener("loadingstatechanged", function (evt) {
                var state = listView.loadingState;

                if (state == "viewPortLoaded") {
                    setImmediate(function () {
                        if (!WinJS.Utilities.hasClass(document.getElementById("waitZone"), "hidden"))
                            WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
                    });
                }
                else if (state == "complete") {
                }
            }, false);

            // Settings
            Windows.Storage.ApplicationData.current.addEventListener("datachanged", onDataChanged);

            // Advanced filters
            if (globalOptions.expansionId != expansion.id) {
                UrzaGatherer.Common.PrepareAdvancedFilters(expansion.cardsList, function () {
                    that.updateLayout(document, appView.value);
                });
            }

            // Advanced filters button
            var filterButton = element.querySelector("#filterButton");
            filterButton.winControl.label = UrzaGatherer.Tools.GetString("AdvancedFilters");

            // Appbar
            var appBarDiv = document.querySelector(".appBar");
            var appBar = appBarDiv.winControl;
            element.querySelector(".cardsList").addEventListener("selectionchanged", function () {
                var count = listView.selection.count();
                if (count > 0) {
                    appBar.showCommands(appBarDiv.querySelectorAll('.select'));
                    appBar.sticky = true;
                    appBar.show();
                } else {
                    appBar.hideCommands(appBarDiv.querySelectorAll('.select'));
                    appBar.hide();
                    appBar.sticky = false;
                }
            });
            appBar.hideCommands(appBarDiv.querySelectorAll('.select'));

            var pinButton = element.querySelector("#pinButton");

            if (Windows.UI.StartScreen.SecondaryTile.exists(expansion.id)) {
                pinButton.winControl.label = UrzaGatherer.Tools.GetString("UnpinLabel");
                pinButton.winControl.icon = WinJS.UI.AppBarIcon.unpin;
                pinButton.winControl.tooltip = UrzaGatherer.Tools.GetString("UnpinTooltip");
            } else {
                pinButton.winControl.label = UrzaGatherer.Tools.GetString("PinLabel");
                pinButton.winControl.icon = WinJS.UI.AppBarIcon.pin;
                pinButton.winControl.tooltip = UrzaGatherer.Tools.GetString("PinTooltip");
            }

            pinButton.addEventListener("click", function () {
                pinButton.winControl.sticky = true;

                if (pinButton.winControl.icon === WinJS.UI.AppBarIcon.pin) {
                    pinByElementAsync(function () {
                        pinButton.winControl.label = UrzaGatherer.Tools.GetString("UnpinLabel");
                        pinButton.winControl.icon = WinJS.UI.AppBarIcon.unpin;
                        pinButton.winControl.tooltip = UrzaGatherer.Tools.GetString("UnpinTooltip");
                    });
                } else {
                    var selectionRect = pinButton.getBoundingClientRect();
                    var tileToGetDeleted = new Windows.UI.StartScreen.SecondaryTile(expansion.id);

                    tileToGetDeleted.requestDeleteAsync({ x: selectionRect.left, y: selectionRect.top }).then(function (isDeleted) {
                        if (isDeleted) {
                            pinButton.winControl.label = UrzaGatherer.Tools.GetString("PinLabel");
                            pinButton.winControl.icon = WinJS.UI.AppBarIcon.pin;
                            pinButton.winControl.tooltip = UrzaGatherer.Tools.GetString("PinTooltip");
                        }
                    });
                };
            }, false);

            // Selection
            var checkButton = element.querySelector("#checkButton");
            checkButton.winControl.label = UrzaGatherer.Tools.GetString("CheckLabel");
            checkButton.winControl.icon = WinJS.UI.AppBarIcon.accept;
            checkButton.winControl.tooltip = UrzaGatherer.Tools.GetString("CheckTooltip");

            checkButton.onclick = function (e) {
                var listView = document.querySelector(".cardsList").winControl;
                var indices = listView.selection.getIndices();

                if (indices.length == 0)
                    return;

                addToCollection(that, indices);
            };

            var checkAllButton = element.querySelector("#checkAllButton");
            checkAllButton.winControl.label = UrzaGatherer.Tools.GetString("CheckAllLabel");
            checkAllButton.winControl.icon = WinJS.UI.AppBarIcon.accept;
            checkAllButton.winControl.tooltip = UrzaGatherer.Tools.GetString("CheckAllTooltip");

            checkAllButton.onclick = function (e) {
                addToCollection(that);
            };

            var uncheckButton = element.querySelector("#uncheckButton");
            uncheckButton.winControl.label = UrzaGatherer.Tools.GetString("UncheckLabel");
            uncheckButton.winControl.icon = WinJS.UI.AppBarIcon.clear;
            uncheckButton.winControl.tooltip = UrzaGatherer.Tools.GetString("UncheckTooltip");

            uncheckButton.onclick = function (e) {
                var listView = document.querySelector(".cardsList").winControl;
                var indices = listView.selection.getIndices();

                if (indices.length == 0)
                    return;

                removeFromCollection(that, indices);
            };

            var uncheckAllButton = element.querySelector("#uncheckAllButton");
            uncheckAllButton.winControl.label = UrzaGatherer.Tools.GetString("UncheckAllLabel");
            uncheckAllButton.winControl.icon = WinJS.UI.AppBarIcon.clear;
            uncheckAllButton.winControl.tooltip = UrzaGatherer.Tools.GetString("UncheckAllTooltip");

            uncheckAllButton.onclick = function (e) {
                removeFromCollection(that);
            };

            // onDataStatusChanged
            UrzaGatherer.Collection.onDataStatusChanged = function () {

                if (UrzaGatherer.UserData) {
                    listView.selectionMode = WinJS.UI.SelectionMode.multi;
                    WinJS.Utilities.removeClass(document.querySelector("#checkFilter"), "hidden");
                }
                else {
                    listView.selectionMode = WinJS.UI.SelectionMode.none;
                    WinJS.Utilities.addClass(document.querySelector("#checkFilter"), "hidden");
                }

                getStats();
            };

            this.updateLayout(element, appView.value);
            listView.element.focus();
        },
        ready: function (element, options) {
            WinJS.Resources.processAll();

            globalOptions = options;

            WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");
            expansion = options.expansion;

            element.querySelector("#pageTitle").innerText = expansion.name;

            if (expansion.isChecked) {
                WinJS.Utilities.addClass(element.querySelector("#cardsCount"), "complete");
            }

            var that = this;

            // Filters
            for (var index = 0; index < filters.length; index++) {
                prepareFilter(filters[index][0], filters[index][1], this);
            }

            // Text filter
            var textFilter = document.getElementById("textFilter");
            textFilter.addEventListener("keyup", function () {
                that.updateLayout(document, appView.value);
            });

            if (globalOptions.expansionTextFilter) {
                document.getElementById("textFilter").value = globalOptions.expansionTextFilter;
            }

            // Check filter
            document.getElementById("checkFilter").addEventListener("change", function () {
                that.updateLayout(document, appView.value);
            });

            if (globalOptions.expansionCheckFilter) {
                document.getElementById("checkFilter").selectedIndex = globalOptions.expansionCheckFilter;
            }

            // Order select
            document.getElementById("orderSelect").addEventListener("change", function () {
                that.updateLayout(document, appView.value);
            });

            if (globalOptions.expansionOrderSelect) {
                document.getElementById("orderSelect").selectedIndex = globalOptions.expansionOrderSelect;
            }
        },

        itemInvoked: function (eventObject) {
            if (UrzaGatherer.FileOpenPickerUI) {
                var item = eventObject.detail.itemPromise.then(function (invokedItem) {
                    var card = invokedItem.data;

                    UrzaGatherer.Tools.GetCardFile(card).then(function (file) {
                        UrzaGatherer.FileOpenPickerUI.addFile(card.name, file);
                    });
                });
                return;
            }

            // Navigating
            nav.navigate("/pages/card/card.html", { cardIndex: eventObject.detail.itemIndex, cards: filtered });
        },

        updateLayout: function (element, viewState, previousViewState) {
            if (previousViewState == viewState)
                return;

            updateSize();

            var listView = element.querySelector(".cardsList").winControl;
            WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");

            // Synchronize filters
            currentFilters = [];
            for (var index = 0; index < filters.length; index++) {
                currentFilters.push(document.getElementById(filters[index][0] + "Filter").value);
            }
            currentTextFilter = UrzaGatherer.Common.GetCurrentTextFilter(document.getElementById("textFilter").value.toLowerCase());
            currentCheckFilter = document.getElementById("checkFilter").selectedIndex;

            currentAdvancedFiltersState = UrzaGatherer.Common.GetCurrentAdvancedFiltersState();

            // Sort and filter
            var sorted = expansion.cardsList.createSorted(UrzaGatherer.Common.SortFunction);
            filtered = sorted.createFiltered(filterFunction);

            getStats();

            if (viewState === appViewState.snapped) {

                listView.itemTemplate = UrzaGatherer.Common.ItemRenderer;
                listView.layout = new ui.ListLayout();

                setImmediate(function () {
                    listView.itemDataSource = filtered.dataSource;
                    listView.groupDataSource = null;
                    if (globalOptions.expansionScrollPosition) {
                        listView.indexOfFirstVisible = globalOptions.expansionScrollPosition;
                        delete globalOptions.expansionScrollPosition;
                    }
                });

            } else {
                listView.layout = new ui.GridLayout({
                    groupHeaderPosition: "top",
                    groupInfo: function () {
                        var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];

                        if (!zoomLevel)
                            zoomLevel = 50;

                        var level = zoomLevel / 100.0;

                        return {
                            enableCellSpanning: true,
                            cellWidth: Math.round(480 * level),
                            cellHeight: Math.round(680 * level)
                        };
                    }
                });
                setImmediate(function () {
                    listView.itemDataSource = filtered.dataSource;
                    if (globalOptions.expansionScrollPosition) {
                        listView.indexOfFirstVisible = globalOptions.expansionScrollPosition;
                        delete globalOptions.expansionScrollPosition;
                    }
                });
            }
        }

    });
})();
