﻿(function () {
    "use strict";

    var appView = Windows.UI.ViewManagement.ApplicationView;
    var appViewState = Windows.UI.ViewManagement.ApplicationViewState;
    var nav = WinJS.Navigation;
    var ui = WinJS.UI;
    var utils = WinJS.Utilities;
    var timer;
    var flipViewDiv;
    var listView;

    var invertTile = function (element) {
        var item = element.querySelector(".item-image-container");
        var backItem = element.querySelector(".item-back");

        if (item == null)
            return;

        WinJS.Utilities.toggleClass(item, "flipped");
        WinJS.Utilities.toggleClass(backItem, "flipped");
    }

    var getStats = function () {
        var stats = UrzaGatherer.GetStatistics();

        document.getElementById("cardsCount").innerText = stats.cards;
        document.getElementById("cardsPrice").innerText = stats.price;
    }

    var saveState = function () {
        // Saving state
        UrzaGatherer.Store.blockScrollPosition = document.querySelector("#blocksList").winControl.indexOfFirstVisible;
    }

    var hideWaitBarOnComplete = function () {
        var state = listView.loadingState;
        if (state == "viewPortLoaded") {
            listView.removeEventListener("loadingstatechanged", hideWaitBarOnComplete);
            setImmediate(function () {
                if (!WinJS.Utilities.hasClass(document.getElementById("waitZone"), "hidden"))
                    WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
            });
        }
        if (state == "complete") {
        }
    }

    ui.Pages.define("/pages/home/home.html", {
        unload: function () {
            window.clearInterval(timer);

            nav.removeEventListener("navigating", saveState);
        },
        sync: function (element) {

            // Search
            if (UrzaGatherer.TextQuery) {
                var textQuery = UrzaGatherer.TextQuery;
                delete UrzaGatherer.TextQuery;

                WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
                nav.navigate("/pages/search/search.html", { textQuery: textQuery });
                return;
            }

            // Secondary tile
            if (UrzaGatherer.Arguments) {
                var expansionID = parseInt(UrzaGatherer.Arguments);
                delete UrzaGatherer.Arguments;

                var expansion;

                for (var index = 0; index < UrzaGatherer.Expansions.length; index++) {
                    var exp = UrzaGatherer.Expansions[index];

                    if (exp.id == expansionID) {
                        expansion = exp;
                        break;
                    }
                }
                WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
                nav.navigate("/pages/expansion/expansion.html", { expansion: expansion });
                return;
            }

            listView = element.querySelector("#blocksList").winControl;

            ui.setOptions(listView, {
                oniteminvoked: this.itemInvoked,
                itemTemplate: this.itemRenderer,
                groupHeaderTemplate: this.headerRenderer,
                layout: new ui.GridLayout({
                    groupHeaderPosition: "top",
                    groupInfo: function () {
                        return {
                            enableCellSpanning: true,
                            cellWidth: 28,
                            cellHeight: 13
                        };
                    }
                })
            });

            ui.setOptions(element.querySelector("#blocksListSnapped").winControl, {
                oniteminvoked: this.itemInvoked.bind(this)
            });

            // Stats
            getStats();

            // Binding
            var that = this;
            setImmediate(function () {
                that.updateLayout(element, appView.value);
            });
        },
        afterPageEnter: function (element) {
            nav.addEventListener("navigating", saveState);

            var that = this;
            var downloadRing = document.querySelector(".downloadRing");
            var progressDiv = document.querySelector(".downloadProgress");

            if (UrzaGatherer.Blocks.length == 0) {
                UrzaGatherer.Init(function () {

                    if (WinJS.Utilities.hasClass(downloadRing, "hidden")) {
                        WinJS.Utilities.removeClass(downloadRing, "hidden");
                        WinJS.Utilities.addClass(progressDiv, "hidden");
                    }

                    if (UrzaGatherer.UserData) {
                        UrzaGatherer.UpdateCardsStatus();

                        getStats();
                    }
                    that.sync(element);

                    UrzaGatherer.Tools.UpdateBackground();

                    UrzaGatherer.Collection.Init();

                }, function (progress) {
                    if (WinJS.Utilities.hasClass(progressDiv, "hidden")) {
                        WinJS.Utilities.removeClass(progressDiv, "hidden");
                        WinJS.Utilities.addClass(downloadRing, "hidden");
                    }

                    progressDiv.value = progress;
                }, function (news) {

                    var currentLanguage = "en";

                    if (Windows.Globalization.Language.currentInputMethodLanguageTag.indexOf("fr-") != -1)
                        currentLanguage = "fr";

                    var messageDialog = new Windows.UI.Popups.MessageDialog(news[currentLanguage], UrzaGatherer.Tools.GetString("WhatsNew"));

                    messageDialog.commands.append(new Windows.UI.Popups.UICommand("OK"));

                    messageDialog.showAsync();
                });
            }

            // Animations
            timer = window.setInterval(function () {
                var index = Math.floor(Math.random() * UrzaGatherer.Expansions.length);
                var item = UrzaGatherer.Expansions[index];

                if (!item || !item.rootElement)
                    return;

                item.isFlipped = !item.isFlipped;

                invertTile(item.rootElement);

                window.setTimeout(function () {
                    item.isFlipped = !item.isFlipped;

                    invertTile(item.rootElement);
                }, 10000);

            }, 1000);
        },
        ready: function (element, options) {
            WinJS.Resources.processAll();

            UrzaGatherer.Collection.onDataStatusChanged = function () {

                if (UrzaGatherer.Blocks.length == 0)
                    return;

                getStats();
            };

            if (UrzaGatherer.Blocks.length !== 0) {
                this.sync(element);
            }
        },

        headerRenderer: function (itemPromise) {
            return itemPromise.then(function (currentItem, recycled) {
                var templateName;

                if (!currentItem.data.logo) {// Expansion
                    templateName = ".cardOfTheDayHeaderTemplate";
                } else { // Card of the day
                    templateName = ".headerTemplate";
                }

                var templateDiv = document.querySelector(templateName);

                if (!templateDiv)
                    return;

                var template = templateDiv.winControl.renderItem(itemPromise, recycled);

                return template.element;
            })
        },

        itemRenderer: function (itemPromise) {
            return itemPromise.then(function (currentItem, recycled) {
                var templateName;

                if (currentItem.data.block) {// Expansion
                    templateName = ".itemTemplate";
                } else { // Card of the day
                    templateName = ".cardOfTheDayTemplate";
                }

                var templateDiv = document.querySelector(templateName);

                if (!templateDiv)
                    return;
                var template = templateDiv.winControl.renderItem(itemPromise, recycled);

                return {
                    element: template.element,
                    renderComplete: template.renderComplete.then(function (elem) {

                        if (!currentItem.data.block) {

                            currentItem.data.element = elem;

                            WinJS.Utilities.addClass(elem, "cardOfTheDay");

                            return;
                        }

                        currentItem.data.rootElement = elem;

                        if (currentItem.data.isFlipped)
                            invertTile(elem);
                    })
                };
            })
        },

        itemInvoked: function (eventObject) {
            var item = eventObject.detail.itemPromise.then(function (invokedItem) {
                if (invokedItem.data.url) { // Card
                    nav.navigate("/pages/card/card.html", { cards: new WinJS.Binding.List([invokedItem.data.card]), cardIndex: 0 });
                } else { // Expansion
                    nav.navigate("/pages/expansion/expansion.html", { expansion: invokedItem.data });
                }
            });
        },

        groupDataSelector: function (item) {
            if (item.block)
                return item.block;

            return { name: UrzaGatherer.Tools.GetString("CardOfTheDay"), banner: "hideBanner" };
        },

        groupKeySelector: function (item) {
            if (item.block) {
                return item.block.index.toString();
            }

            // Card of the day
            return "1000000";
        },

        groupCompare: function (i0, i1) {
            var index0 = parseInt(i0);
            var index1 = parseInt(i1);

            return index1 - index0;
        },

        updateLayout: function (element, viewState, previousViewState) {
            if (previousViewState == viewState)
                return;

            var fullModeDiv = element.querySelector(".fullMode");
            var listViewSnapped = element.querySelector("#blocksListSnapped");

            if (viewState === appViewState.snapped) {
                var filtered = UrzaGatherer.ExpansionsList.createSorted(function (i0, i1) {
                    var e0 = i0.block.index;
                    var e1 = i1.block.index;

                    if (e0 != e1)
                        return e1 - e0;

                    return i1.orderInBlock - i0.orderInBlock;
                });

                listViewSnapped.winControl.itemTemplate = this.itemRenderer;
                listViewSnapped.winControl.itemDataSource = filtered.dataSource;

                if (!WinJS.Utilities.hasClass(fullModeDiv, "hidden"))
                    WinJS.Utilities.addClass(fullModeDiv, "hidden");
                if (WinJS.Utilities.hasClass(listViewSnapped, "hidden"))
                    WinJS.Utilities.removeClass(listViewSnapped, "hidden");
            } else {
                if (WinJS.Utilities.hasClass(document.getElementById("waitZone"), "hidden"))
                    WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");

                listView.addEventListener("loadingstatechanged", hideWaitBarOnComplete, false);

                var zoomedListView = element.querySelector(".zoomedList").winControl;

                var groupDataSource = UrzaGatherer.ExpansionsListWithCardOfTheDay.createGrouped(this.groupKeySelector, this.groupDataSelector, this.groupCompare);

                listView.itemDataSource = groupDataSource.dataSource;
                listView.groupDataSource = groupDataSource.groups.dataSource;
                zoomedListView.itemDataSource = groupDataSource.groups.dataSource;

                if (UrzaGatherer.Store.blockScrollPosition != undefined) {
                    //     listView.indexOfFirstVisible = UrzaGatherer.Store.blockScrollPosition;
                    delete UrzaGatherer.Store.blockScrollPosition;
                }
                else
                    element.querySelector(".zoomControl").winControl.forceLayout();

                if (WinJS.Utilities.hasClass(fullModeDiv, "hidden"))
                    WinJS.Utilities.removeClass(fullModeDiv, "hidden");
                if (!WinJS.Utilities.hasClass(listViewSnapped, "hidden"))
                    WinJS.Utilities.addClass(listViewSnapped, "hidden");
            }
        },

    });
})();
