﻿(function () {
    "use strict";

    var appView = Windows.UI.ViewManagement.ApplicationView;
    var appViewState = Windows.UI.ViewManagement.ApplicationViewState;
    var nav = WinJS.Navigation;
    var ui = WinJS.UI;
    var utils = WinJS.Utilities;
    var all = UrzaGatherer.Tools.GetString("All");

    var textQuery;
    var authorQuery;
    var rarityQuery;
    var colorQuery;
    var filters = [["color", UrzaGatherer.Tools.GetString("Colors")], ["author", UrzaGatherer.Tools.GetString("Authors")], ["rarity", UrzaGatherer.Tools.GetString("Rarities")], ["type", UrzaGatherer.Tools.GetString("Types")]];
    var filtered;
    var cards;
    var globalOptions;

    var currentAdvancedFiltersState;

    var currentFilters;
    var currentCheckFilter;
    var currentTextFilter;

    var updateSize = function (forceUpdate) {
        var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];

        if (!zoomLevel)
            return;

        var css = UrzaGatherer.Tools.FindCSS("search.css");
        var level = zoomLevel / 100.0;

        for (var index = 0; index < css.cssRules.length; index++) {
            if (css.cssRules[index].selectorText == ".search .cardsList .item-container") {
                css.cssRules[index].style.width = (480 * level) + "px";
                css.cssRules[index].style.height = (680 * level) + "px";
            }
        }

        if (forceUpdate) {
            var listView = document.querySelector(".cardsList").winControl;
            listView.forceLayout();
        }
    }

    var sortFunction = function (i0, i1) {
        return i0.name > i1.name;
    }

    var filterFunction = function (item) {

        //Filters
        for (var index = 0; index < filters.length; index++) {
            var filter = currentFilters[index];

            if (filter.substring(0, all.length) != all) {
                if (!(item[filters[index][0]] == filter))
                    return false;
            }
        }

        // Text
        if (!(item.name.toLowerCase().indexOf(textQuery) != -1
                || (item.text && item.text.toLowerCase().indexOf(textQuery) != -1)
                || (item.flavor && item.flavor.toLowerCase().indexOf(textQuery) != -1)
                || (item.type && item.type.toLowerCase().indexOf(textQuery) != -1)
                )) {
            return false;
        }

        // Text
        if (!UrzaGatherer.Common.CheckTextFilter(currentTextFilter, item))
            return false;

        // Check
        if (UrzaGatherer.UserData) {
            if (currentCheckFilter > 0) {
                if (!(currentCheckFilter == 1 && !item.isChecked) || (currentCheckFilter == 2 && item.isChecked))
                    return false;
            }
        }

        // Advanced filters
        if (currentAdvancedFiltersState.manaCostFilterActivated) {
            if (!(item.convertedManaCost >= currentAdvancedFiltersState.currentMinManaCost && item.convertedManaCost <= currentAdvancedFiltersState.currentMaxManaCost))
                return false;
        }

        if (currentAdvancedFiltersState.priceFilterActivated) {
            if (!(item.price !== undefined && item.price >= currentAdvancedFiltersState.currentMinPrice && item.price <= currentAdvancedFiltersState.currentMaxPrice))
                return false;
        }

        if (currentAdvancedFiltersState.forceFilterActivated) {
            if (!(item.force !== undefined && item.force >= currentAdvancedFiltersState.currentMinForce && item.force <= currentAdvancedFiltersState.currentMaxForce))
                return false;
        }

        if (currentAdvancedFiltersState.defenseFilterActivated) {
            if (!(item.defense !== undefined && item.defense >= currentAdvancedFiltersState.currentMinDefense && item.defense <= currentAdvancedFiltersState.currentMaxDefense))
                return false;
        }

        return true;
    }

    var prepareFilter = function (property, plural, source, that) {
        var filter = document.getElementById(property + "Filter");
        var results = [];

        for (var cardIndex = 0; cardIndex < source.length; cardIndex++) {
            var value = source.getAt(cardIndex)[property];

            if (results.indexOf(value) == -1)
                results.push(value);
        }
        var allLength = all.length;

        results.push(all + plural);
        var sortedResults = results.sort(function (i0, i1) {
            if (i0 == i1)
                return 0;

            if (i0.substring(0, allLength) == all)
                return -1;

            if (i1.substring(0, allLength) == all)
                return 1;

            if (i0 > i1)
                return 1;

            return -1;
        });

        var selectIndex = 0;

        for (var index = 0; index < sortedResults.length; index++) {
            filter.options[index] = new Option(sortedResults[index]);

            var query = globalOptions[property + "Query"];

            if (query && sortedResults[index].toLowerCase() == query) {
                selectIndex = index;
            }

            var stored = globalOptions["search" + property + "Filter"];

            if (stored && stored == sortedResults[index]) {
                selectIndex = index;
                delete globalOptions["search" + property + "Filter"];
            }
        }

        filter.selectedIndex = selectIndex;

        filter.addEventListener("change", function () {
            that.updateLayout(document, appView.value);
        });
    };

    var queryFunction = function (item) {
        var result = true;

        if (textQuery != "") {
            if (!(item.name.toLowerCase().indexOf(textQuery) != -1
                || (item.text && item.text.toLowerCase().indexOf(textQuery) != -1)
                || (item.flavor && item.flavor.toLowerCase().indexOf(textQuery) != -1)
                || (item.type && item.type.toLowerCase().indexOf(textQuery) != -1)
                )) {
                return false;
            }
        }

        if (authorQuery != "") {
            if (!(item.author.toLowerCase().indexOf(authorQuery) != -1)) {
                return false;
            }
        }

        if (rarityQuery != "") {
            if (!(item.rarity.toLowerCase().indexOf(rarityQuery) != -1)) {
                return false;
            }
        }

        if (colorQuery != "") {
            if (!(item.color.toLowerCase().indexOf(colorQuery) != -1)) {
                return false;
            }
        }

        return true;
    };

    var saveState = function () {
        var listView = document.querySelector(".cardsList").winControl;

        // Saving state
        for (var index = 0; index < filters.length; index++) {
            globalOptions["search" + filters[index][0] + "Filter"] = document.getElementById(filters[index][0] + "Filter").value;
        }

        globalOptions.searchTextFilter = document.getElementById("textFilter").value;
        globalOptions.searchCheckFilter = document.getElementById("checkFilter").selectedIndex;
        globalOptions.searchScrollPosition = listView.indexOfFirstVisible;
    }

    var onUpdateSizechanged = function () {
        updateSize(true);
    }

    ui.Pages.define("/pages/search/search.html", {
        unload: function () {
            nav.removeEventListener("navigating", saveState);

            var bar = document.querySelector(".appBar");
            bar.parentNode.removeChild(bar);

            Windows.Storage.ApplicationData.current.removeEventListener("datachanged", onUpdateSizechanged);
        },
        afterPageEnter: function (element) {

            nav.addEventListener("navigating", saveState);

            var that = this;
            // Advanced filters button
            var filterButton = element.querySelector("#filterButton");
            filterButton.winControl.label = UrzaGatherer.Tools.GetString("AdvancedFilters");

            var listView = element.querySelector(".cardsList").winControl;

            ui.setOptions(listView, {
                itemTemplate: UrzaGatherer.Common.ItemRenderer,
                oniteminvoked: that.itemInvoked
            });

            if (UrzaGatherer.UserData) {
                WinJS.Utilities.removeClass(document.querySelector("#checkFilter"), "hidden");
            }

            listView.addEventListener("loadingstatechanged", function (evt) {
                var state = listView.loadingState;

                if (state == "viewPortLoaded") {
                    setImmediate(function () {
                        if (!WinJS.Utilities.hasClass(document.getElementById("waitZone"), "hidden"))
                            WinJS.Utilities.addClass(document.getElementById("waitZone"), "hidden");
                    });
                }
                else if (state == "complete") {
                }
            }, false);

            that.updateLayout(element, appView.value);

            // onDataStatusChanged
            UrzaGatherer.Collection.onDataStatusChanged = function () {
                if (UrzaGatherer.UserData)
                    WinJS.Utilities.removeClass(document.querySelector("#checkFilter"), "hidden");
                else
                    WinJS.Utilities.addClass(document.querySelector("#checkFilter"), "hidden");
            };
        },
        ready: function (element, options) {
            WinJS.Resources.processAll();

            textQuery = options.textQuery || "";
            authorQuery = options.authorQuery || "";
            rarityQuery = options.rarityQuery || "";
            colorQuery = options.colorQuery || "";
            globalOptions = options;

            cards = new WinJS.Binding.List(UrzaGatherer.Cards.filter(queryFunction));

            var that = this;

            // Advanced filters
            UrzaGatherer.Common.PrepareAdvancedFilters(cards, function () {
                that.updateLayout(document, appView.value);
            });

            // Filters
            for (var index = 0; index < filters.length; index++) {
                prepareFilter(filters[index][0], filters[index][1], cards, that);
            }

            // Settings
            Windows.Storage.ApplicationData.current.addEventListener("datachanged", onUpdateSizechanged);

            // Text filter
            document.getElementById("textFilter").addEventListener("keyup", function () {
                that.updateLayout(document, appView.value);
            });

            if (globalOptions.searchTextFilter) {
                document.getElementById("textFilter").value = globalOptions.searchTextFilter;
            }
            // Check filter
            document.getElementById("checkFilter").addEventListener("change", function () {
                that.updateLayout(document, appView.value);
            });

            if (globalOptions.searchCheckFilter) {
                document.getElementById("checkFilter").selectedIndex = globalOptions.searchCheckFilter;
                delete globalOptions.searchCheckFilter;
            }

            WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");
        },

        itemInvoked: function (eventObject) {
            var listView = document.querySelector(".cardsList").winControl;

            nav.navigate("/pages/card/card.html", { cardIndex: eventObject.detail.itemIndex, cards: filtered });
        },

        updateLayout: function (element, viewState, previousViewState) {
            if (previousViewState == viewState)
                return;

            updateSize();

            var listView = element.querySelector(".cardsList").winControl;
            WinJS.Utilities.removeClass(document.getElementById("waitZone"), "hidden");

            // Synchronize filters
            currentFilters = [];
            for (var index = 0; index < filters.length; index++) {
                currentFilters.push(document.getElementById(filters[index][0] + "Filter").value);
            }
            currentTextFilter = UrzaGatherer.Common.GetCurrentTextFilter(document.getElementById("textFilter").value.toLowerCase());
            currentCheckFilter = document.getElementById("checkFilter").selectedIndex;

            currentAdvancedFiltersState = UrzaGatherer.Common.GetCurrentAdvancedFiltersState();

            // Sort and filter
            var sorted = cards.createSorted(sortFunction);
            filtered = sorted.createFiltered(filterFunction);

            // Price
            var count = filtered.length;
            var price = 0;

            for (var index = 0; index < count; index++) {
                var card = filtered.getAt(index);
                price += card.price;
            }

            document.getElementById("cardsCount").innerText = count + UrzaGatherer.Tools.GetString("Cards");
            document.getElementById("cardsPrice").innerText = Math.round(price * 100) / 100 + "€";

            // Binding
            if (viewState === appViewState.snapped) {

                listView.itemTemplate = UrzaGatherer.Common.ItemRenderer;
                listView.layout = new ui.ListLayout();

                setImmediate(function () {
                    listView.itemDataSource = filtered.dataSource;
                    listView.groupDataSource = null;
                    if (globalOptions.searchScrollPosition) {
                        listView.indexOfFirstVisible = globalOptions.searchScrollPosition;
                        delete globalOptions.searchScrollPosition;
                    }
                });

            } else {
                listView.layout = new ui.GridLayout({
                    groupHeaderPosition: "top",
                    groupInfo: function () {
                        var zoomLevel = Windows.Storage.ApplicationData.current.localSettings.values["zoomLevel"];

                        if (!zoomLevel)
                            zoomLevel = 50;

                        var level = zoomLevel / 100.0;

                        return {
                            enableCellSpanning: true,
                            cellWidth: Math.round(480 * level),
                            cellHeight: Math.round(680 * level)
                        };
                    }
                });
                setImmediate(function () {
                    listView.itemDataSource = filtered.dataSource;
                    if (globalOptions.searchScrollPosition) {
                        listView.indexOfFirstVisible = globalOptions.searchScrollPosition;
                        delete globalOptions.searchScrollPosition;
                    }
                });
            }
        }

    });
})();
